# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'llm_preferences.ui'
##
## Created by: Qt User Interface Compiler version 6.9.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QGridLayout, QGroupBox,
    QHBoxLayout, QLabel, QLineEdit, QPlainTextEdit,
    QPushButton, QSizePolicy, QSplitter, QVBoxLayout,
    QWidget)

class Ui_llm_preferences_widget(object):
    def setupUi(self, llm_preferences_widget):
        if not llm_preferences_widget.objectName():
            llm_preferences_widget.setObjectName(u"llm_preferences_widget")
        llm_preferences_widget.resize(1371, 957)
        self.gridLayout = QGridLayout(llm_preferences_widget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.splitter = QSplitter(llm_preferences_widget)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Vertical)
        self.groupBox = QGroupBox(self.splitter)
        self.groupBox.setObjectName(u"groupBox")
        self.gridLayout_3 = QGridLayout(self.groupBox)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.prefix = QPlainTextEdit(self.groupBox)
        self.prefix.setObjectName(u"prefix")

        self.gridLayout_3.addWidget(self.prefix, 0, 0, 1, 1)

        self.splitter.addWidget(self.groupBox)
        self.groupBox_2 = QGroupBox(self.splitter)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.gridLayout_4 = QGridLayout(self.groupBox_2)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.suffix = QPlainTextEdit(self.groupBox_2)
        self.suffix.setObjectName(u"suffix")

        self.gridLayout_4.addWidget(self.suffix, 0, 0, 1, 1)

        self.splitter.addWidget(self.groupBox_2)

        self.gridLayout.addWidget(self.splitter, 0, 0, 1, 1)

        self.groupBox_3 = QGroupBox(llm_preferences_widget)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.horizontalLayout_5 = QHBoxLayout(self.groupBox_3)
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.label = QLabel(self.groupBox_3)
        self.label.setObjectName(u"label")

        self.verticalLayout_2.addWidget(self.label)

        self.botname = QLineEdit(self.groupBox_3)
        self.botname.setObjectName(u"botname")
        self.botname.setCursorPosition(6)

        self.verticalLayout_2.addWidget(self.botname)


        self.horizontalLayout_5.addLayout(self.verticalLayout_2)

        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.label_2 = QLabel(self.groupBox_3)
        self.label_2.setObjectName(u"label_2")

        self.verticalLayout_3.addWidget(self.label_2)

        self.personality_type = QComboBox(self.groupBox_3)
        self.personality_type.addItem("")
        self.personality_type.addItem("")
        self.personality_type.addItem("")
        self.personality_type.addItem("")
        self.personality_type.addItem("")
        self.personality_type.setObjectName(u"personality_type")

        self.verticalLayout_3.addWidget(self.personality_type)


        self.horizontalLayout_5.addLayout(self.verticalLayout_3)


        self.gridLayout.addWidget(self.groupBox_3, 1, 0, 1, 1)

        self.groupBox_4 = QGroupBox(llm_preferences_widget)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.gridLayout_13 = QGridLayout(self.groupBox_4)
        self.gridLayout_13.setObjectName(u"gridLayout_13")
        self.username = QLineEdit(self.groupBox_4)
        self.username.setObjectName(u"username")

        self.gridLayout_13.addWidget(self.username, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.groupBox_4, 2, 0, 1, 1)

        self.generate_characters_button = QPushButton(llm_preferences_widget)
        self.generate_characters_button.setObjectName(u"generate_characters_button")

        self.gridLayout.addWidget(self.generate_characters_button, 3, 0, 1, 1)

        QWidget.setTabOrder(self.prefix, self.suffix)
        QWidget.setTabOrder(self.suffix, self.botname)
        QWidget.setTabOrder(self.botname, self.username)

        self.retranslateUi(llm_preferences_widget)
        self.generate_characters_button.clicked.connect(llm_preferences_widget.action_button_clicked_generate_characters)
        self.username.textEdited.connect(llm_preferences_widget.username_text_changed)
        self.botname.textEdited.connect(llm_preferences_widget.botname_text_changed)
        self.personality_type.textHighlighted.connect(llm_preferences_widget.personality_type_changed)
        self.suffix.textChanged.connect(llm_preferences_widget.suffix_text_changed)
        self.prefix.textChanged.connect(llm_preferences_widget.prefix_text_changed)

        QMetaObject.connectSlotsByName(llm_preferences_widget)
    # setupUi

    def retranslateUi(self, llm_preferences_widget):
        llm_preferences_widget.setWindowTitle(QCoreApplication.translate("llm_preferences_widget", u"Form", None))
        self.groupBox.setTitle(QCoreApplication.translate("llm_preferences_widget", u"Prefix", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("llm_preferences_widget", u"Suffix", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("llm_preferences_widget", u"Bot details", None))
        self.label.setText(QCoreApplication.translate("llm_preferences_widget", u"Name", None))
        self.botname.setText(QCoreApplication.translate("llm_preferences_widget", u"ChatAI", None))
        self.label_2.setText(QCoreApplication.translate("llm_preferences_widget", u"Personality", None))
        self.personality_type.setItemText(0, QCoreApplication.translate("llm_preferences_widget", u"Nice", None))
        self.personality_type.setItemText(1, QCoreApplication.translate("llm_preferences_widget", u"Mean", None))
        self.personality_type.setItemText(2, QCoreApplication.translate("llm_preferences_widget", u"Weird", None))
        self.personality_type.setItemText(3, QCoreApplication.translate("llm_preferences_widget", u"Insane", None))
        self.personality_type.setItemText(4, QCoreApplication.translate("llm_preferences_widget", u"Random", None))

        self.groupBox_4.setTitle(QCoreApplication.translate("llm_preferences_widget", u"User name", None))
        self.username.setText(QCoreApplication.translate("llm_preferences_widget", u"User", None))
        self.generate_characters_button.setText(QCoreApplication.translate("llm_preferences_widget", u"Generate Characters", None))
    # retranslateUi




from sqlalchemy import Column, Integer, String, Float, JSON

from airunner.data.models.base import BaseModel


class LanguageSettings(BaseModel):
    __tablename__ = "language_settings"
    id = Column(Integer, primary_key=True, autoincrement=True)
    gui_language = Column(String, nullable=True)
    user_language = Column(String, nullable=True)
    bot_language = Column(String, nullable=True)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'metadata.ui'
##
## Created by: Qt User Interface Compiler version 6.9.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QGridLayout, QLabel,
    QRadioButton, QSizePolicy, QSpacerItem, QWidget)

class Ui_metadata_setup(object):
    def setupUi(self, metadata_setup):
        if not metadata_setup.objectName():
            metadata_setup.setObjectName(u"metadata_setup")
        metadata_setup.resize(566, 375)
        self.gridLayout = QGridLayout(metadata_setup)
        self.gridLayout.setObjectName(u"gridLayout")
        self.radioButton = QRadioButton(metadata_setup)
        self.radioButton.setObjectName(u"radioButton")
        self.radioButton.setChecked(False)

        self.gridLayout.addWidget(self.radioButton, 4, 0, 1, 1)

        self.label = QLabel(metadata_setup)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        self.label.setFont(font)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.label_3 = QLabel(metadata_setup)
        self.label_3.setObjectName(u"label_3")

        self.gridLayout.addWidget(self.label_3, 3, 0, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 6, 0, 1, 1)

        self.line = QFrame(metadata_setup)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line, 2, 0, 1, 1)

        self.label_2 = QLabel(metadata_setup)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setWordWrap(True)

        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)

        self.radioButton_2 = QRadioButton(metadata_setup)
        self.radioButton_2.setObjectName(u"radioButton_2")
        self.radioButton_2.setChecked(True)

        self.gridLayout.addWidget(self.radioButton_2, 5, 0, 1, 1)


        self.retranslateUi(metadata_setup)
        self.radioButton.toggled.connect(metadata_setup.yes_toggled)
        self.radioButton_2.toggled.connect(metadata_setup.no_toggled)

        QMetaObject.connectSlotsByName(metadata_setup)
    # setupUi

    def retranslateUi(self, metadata_setup):
        metadata_setup.setWindowTitle(QCoreApplication.translate("metadata_setup", u"Form", None))
        self.radioButton.setText(QCoreApplication.translate("metadata_setup", u"Yes", None))
        self.label.setText(QCoreApplication.translate("metadata_setup", u"Metadata Setup", None))
        self.label_3.setText(QCoreApplication.translate("metadata_setup", u"Would you like to enable metadata in images?", None))
        self.label_2.setText(QCoreApplication.translate("metadata_setup", u"Metadata is attached to PNGs in order to help facilitate the creation of similar images. You may adjust this setting in the preferences of the main application.", None))
        self.radioButton_2.setText(QCoreApplication.translate("metadata_setup", u"No", None))
    # retranslateUi




from airunner.settings import AIRUNNER_ART_ENABLED


ai_art_models = [
    {
        "name": "Stable Diffusion 1.5",
        "path": "stable-diffusion-v1-5/stable-diffusion-v1-5",
        "branch": "main",
        "version": "SD 1.5",
        "category": "stablediffusion",
        "pipeline_action": "txt2img",
        "enabled": True,
        "model_type": "art",
        "is_default": True,
    },
    {
        "name": "Stable Diffusion XL Base 1.0",
        "path": "stabilityai/stable-diffusion-xl-base-1.0",
        "branch": "main",
        "version": "SDXL 1.0",
        "category": "stablediffusion",
        "pipeline_action": "txt2img",
        "enabled": True,
        "model_type": "art",
        "is_default": True,
    },
    {
        "name": "Stable Diffusion XL Turbo",
        "path": "stabilityai/stable-diffusion-xl-base-1.0",
        "branch": "main",
        "version": "SDXL Turbo",
        "category": "stablediffusion",
        "pipeline_action": "txt2img",
        "enabled": True,
        "model_type": "art",
        "is_default": True,
    },
    {
        "name": "Stable Diffusion Inpaint 1.5",
        "path": "stable-diffusion-v1-5/stable-diffusion-inpainting",
        "branch": "fp16",
        "version": "SD 1.5",
        "category": "stablediffusion",
        "pipeline_action": "inpaint",
        "enabled": True,
        "model_type": "art",
        "is_default": True,
    },
    {
        "name": "SDXL Inpaint",
        "path": "diffusers/stable-diffusion-xl-1.0-inpainting-0.1",
        "branch": "fp16",
        "version": "SDXL 1.0",
        "category": "stablediffusion",
        "pipeline_action": "inpaint",
        "enabled": True,
        "model_type": "art",
        "is_default": True,
    },
    {
        "name": "CompVis Safety Checker",
        "path": "CompVis/stable-diffusion-safety-checker",
        "branch": "main",
        "version": "SD 1.5",
        "category": "stablediffusion",
        "pipeline_action": "txt2img",
        "enabled": True,
        "model_type": "art",
        "is_default": True,
    },
    {
        "name": "OpenAI Feature Extractor",
        "path": "openai/clip-vit-large-patch14",
        "branch": "main",
        "version": "SD 1.5",
        "category": "stablediffusion",
        "pipeline_action": "txt2img",
        "enabled": True,
        "model_type": "art",
        "is_default": True,
    },
    {
        "name": "Inpaint vae",
        "path": "cross-attention/asymmetric-autoencoder-kl-x-1-5",
        "branch": "fp16",
        "version": "SD 1.5",
        "category": "stablediffusion",
        "pipeline_action": "inpaint_vae",
        "enabled": True,
        "model_type": "art",
        "is_default": True,
    },
    {
        "name": "Flux S",
        "path": "black-forest-labs/FLUX.1-schnell",
        "branch": "main",
        "version": "Flux S",
        "category": "flux",
        "pipeline_action": "txt2img",
        "enabled": True,
        "model_type": "art",
        "is_default": True,
    },
]

llm_models = [
    {
        "name": "Mistral 7B Instruct v3",
        "path": "w4ffl35/Ministral-8B-Instruct-2410-doublequant",
        "branch": "main",
        "version": "0.3",
        "category": "llm",
        "pipeline_action": "causallm",
        "enabled": True,
        "model_type": "llm",
        "is_default": True,
    },
    {
        "name": "Intfloat E5 Large",
        "path": "intfloat/e5-large",
        "branch": "main",
        "version": "llm",
        "category": "llm",
        "pipeline_action": "embedding",
        "enabled": True,
        "model_type": "llm",
        "is_default": True,
    },
]

if AIRUNNER_ART_ENABLED:
    model_bootstrap_data = ai_art_models + llm_models
else:
    model_bootstrap_data = llm_models



from typing import Dict
from PIL.Image import Image
from PIL import ImageQt
from PySide6.QtGui import QPainter, QColor, Qt

from airunner.enums import CanvasToolName, SignalCode
from airunner.utils.image import (
    convert_binary_to_image,
    convert_image_to_binary,
)
from airunner.gui.widgets.canvas.brush_scene import BrushScene


class InputImageScene(BrushScene):
    """Scene for handling drawing on input images."""

    def __init__(
        self, canvas_type: str, settings_key: str, is_mask: bool = False
    ):
        super().__init__(canvas_type)
        self._settings_key = settings_key
        self._is_mask = is_mask

    @property
    def settings_key(self):
        return self._settings_key

    @property
    def current_active_image(self) -> Image:
        if self._is_mask:
            # For mask image (outpainting mask)
            base_64_image = self.drawing_pad_settings.mask
        elif (
            self.settings_key == "controlnet_settings"
            and hasattr(self, "use_generated_image")
            and self.use_generated_image
        ):
            # For controlnet generated image
            base_64_image = self.controlnet_settings.generated_image
        elif self.settings_key == "outpaint_settings":
            # For outpaint image
            base_64_image = self.outpaint_settings.image
        elif self.settings_key == "image_to_image_settings":
            # For image-to-image
            base_64_image = self.image_to_image_settings.image
        else:
            # Default to drawing pad image
            base_64_image = self.current_settings.image

        try:
            return convert_binary_to_image(base_64_image)
        except Exception:
            return None

    @current_active_image.setter
    def current_active_image(self, image: Image):
        if image is not None:
            image_binary = convert_image_to_binary(image)

            if self._is_mask:
                # For mask image
                self.update_drawing_pad_settings("mask", image_binary)
            elif (
                self.settings_key == "controlnet_settings"
                and hasattr(self, "use_generated_image")
                and self.use_generated_image
            ):
                # For controlnet generated image
                self.update_controlnet_settings(
                    "generated_image", image_binary
                )
            elif self.settings_key == "outpaint_settings":
                # For outpaint image
                self.update_outpaint_settings("image", image_binary)
            elif self.settings_key == "image_to_image_settings":
                # For image-to-image
                self.update_image_to_image_settings("image", image_binary)
            else:
                # Update the appropriate settings
                self._update_current_settings("image", image_binary)

    def _handle_left_mouse_release(self, event) -> bool:
        self.draw_button_down = False

        # Get the correct image based on our settings
        if self.is_brush_or_eraser:
            # Convert image for saving
            image = ImageQt.fromqimage(self.active_image)
            base_64_image = convert_image_to_binary(image)

            # Update based on what type of input image this is
            if self._is_mask:
                # For mask image
                self.update_drawing_pad_settings("mask", base_64_image)
                # Also update the database model
                model = self.drawing_pad_settings.__class__.objects.first()
                model.mask = base_64_image
                model.save()
            elif (
                self.settings_key == "controlnet_settings"
                and hasattr(self, "use_generated_image")
                and self.use_generated_image
            ):
                # For controlnet generated image
                self.update_controlnet_settings(
                    "generated_image", base_64_image
                )
                model = self.controlnet_settings.__class__.objects.first()
                model.generated_image = base_64_image
                model.save()
            elif self.settings_key == "outpaint_settings":
                # For outpaint image
                self.update_outpaint_settings("image", base_64_image)
                model = self.outpaint_settings.__class__.objects.first()
                model.image = base_64_image
                model.save()
            elif self.settings_key == "image_to_image_settings":
                # For image-to-image
                self.update_image_to_image_settings("image", base_64_image)
                model = self.image_to_image_settings.__class__.objects.first()
                model.image = base_64_image
                model.save()

            if self.drawing_pad_settings.enable_automatic_drawing:
                self.api.art.send_request()

        # Clear drawing state
        self._is_drawing = False
        self._is_erasing = False

        return True  # Event handled

    def _handle_image_generated_signal(self, data: Dict):
        if self.current_settings.lock_input_image:
            return
        super()._handle_image_generated_signal(data)



from PySide6.QtCore import Slot
import sounddevice as sd
from airunner.gui.widgets.base_widget import BaseWidget
from airunner.gui.widgets.sound_settings.templates.sound_settings_ui import (
    Ui_SoundSettings,
)
from airunner.data.models import SoundSettings
from airunner.enums import SignalCode


class SoundSettingsWidget(BaseWidget):
    widget_class_ = Ui_SoundSettings

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_devices()  # Ensure devices are loaded on initialization
        self.monitoring = True

    def load_devices(self):
        # Populate comboboxes with available audio devices
        devices = sd.query_devices()
        output_devices = filter(
            None,
            [
                None if device["max_output_channels"] == 0 else device
                for device in devices
            ],
        )
        input_devices = filter(
            None,
            [
                None if device["max_input_channels"] == 0 else device
                for device in devices
            ],
        )

        current_output_device = None
        current_input_device = None
        
        try:
            current_output_device = sd.query_devices(kind="output")
        except sd.PortAudioError as e:
            self.logger.error(f"PortAudioError: Unable to query output devices. {e}")
        
        try:
            current_input_device = sd.query_devices(kind="input")
        except sd.PortAudioError as e:
            self.logger.error(f"PortAudioError: Unable to query input devices. {e}")

        input_device_names = [device["name"] for device in input_devices]
        output_device_names = [device["name"] for device in output_devices]

        self.ui.playbackComboBox.clear()
        self.ui.playbackComboBox.addItems(output_device_names)
        
        # Set current device only if successfully queried
        if current_output_device is not None and output_device_names:
            self.ui.playbackComboBox.setCurrentText(current_output_device["name"])

        self.ui.recordingComboBox.clear()
        self.ui.recordingComboBox.addItems(input_device_names)
        
        # Set current device only if successfully queried
        if current_input_device is not None and input_device_names:
            self.ui.recordingComboBox.setCurrentText(current_input_device["name"])

        # Retrieve current settings
        sound_settings = SoundSettings.objects.first()

        # Set playback device
        if (
            sound_settings
            and sound_settings.playback_device in output_device_names
        ):
            self.ui.playbackComboBox.setCurrentText(
                sound_settings.playback_device
            )
        elif output_device_names:
            current_device = output_device_names[0]
            self.ui.playbackComboBox.setCurrentIndex(0)
            sound_settings = SoundSettings.objects.first()
            if sound_settings:
                SoundSettings.objects.update(
                    sound_settings.id, playback_device=current_device
                )

        # Set recording device
        if (
            sound_settings
            and sound_settings.recording_device in input_device_names
        ):
            self.ui.recordingComboBox.setCurrentText(
                sound_settings.recording_device
            )
        elif input_device_names:
            current_device = input_device_names[0]
            self.ui.recordingComboBox.setCurrentIndex(0)
            sound_settings = SoundSettings.objects.first()
            if sound_settings:
                SoundSettings.objects.update(
                    sound_settings.id, recording_device=current_device
                )

    @Slot(str)
    def on_playbackComboBox_currentTextChanged(self, device: str):
        sound_settings = SoundSettings.objects.first()
        if sound_settings is None:
            SoundSettings.objects.create(
                playback_device=device, recording_device=None
            )
        sound_settings = SoundSettings.objects.first()
        SoundSettings.objects.update(sound_settings.id, playback_device=device)
        self.emit_signal(SignalCode.PLAYBACK_DEVICE_CHANGED, device)

    @Slot(str)
    def on_recordingComboBox_currentTextChanged(self, device: str):
        sound_settings = SoundSettings.objects.first()
        if sound_settings is None:
            SoundSettings.objects.create(
                recording_device=device, playback_device=None
            )
            sound_settings = SoundSettings.objects.first()
        SoundSettings.objects.update(
            sound_settings.id, recording_device=device
        )
        self.emit_signal(SignalCode.RECORDING_DEVICE_CHANGED, device)

    def update_microphone_volume(self, volume):
        sound_settings = SoundSettings.objects.first()
        if sound_settings is None:
            SoundSettings.objects.create(
                microphone_volume=volume, playback_device=None, recording_device=None
            )
            sound_settings = SoundSettings.objects.first()
        SoundSettings.objects.update(
            sound_settings.id, microphone_volume=volume
        )

    def adjust_input_level(self, value):
        # Adjust the microphone input level
        print(f"Input level adjusted to: {value}")
        # Replace with actual logic to adjust microphone input level



from typing import List, Type
from airunner.handlers.llm.agent.actions.agent_action import AgentAction
from airunner.handlers.llm.agent.actions.agent_action_message import (
    AgentActionMessage,
)
from airunner.utils.application.get_logger import get_logger


class WorkFlow:
    @staticmethod
    def run(
        name: str,
        description: str,
        message: AgentActionMessage,
        steps: List[Type[AgentAction]],
    ):
        logger = get_logger(__name__)
        logger.info(f"Running workflow: {name}")
        logger.info(f"Description: {description}")
        current_message = message
        for current_step in steps:
            logger.info(f"Running step: {current_step.__name__}")
            current_message = current_step.run(current_message)



import subprocess


def main():
    for lang in [
        "english",
        "japanese",
    ]:
        subprocess.run(
            [
                "pyside6-lrelease",
                f"./src/airunner/translations/{lang}.ts",
                "-qm",
                f"./src/airunner/translations/{lang}.qm",
            ],
            check=True,
        )


if __name__ == "__main__":
    main()



FLUX_FILE_BOOTSTRAP_DATA = {
    "Flux S": [
        "model_index.json",
        "scheduler/scheduler_config.json",
        "text_encoder/config.json",
        "text_encoder_2/config.json",
        "text_encoder_2/model.safetensors.index.json",
        "tokenizer/merges.txt",
        "tokenizer/special_tokens_map.json",
        "tokenizer_config.json",
        "tokenizer/vocab.json",
        "tokenizer_2/special_tokens_map.json",
        "tokenizer_2/tokenizer_config.json",
        "transformer/config.json",
        "vae/config.json",
    ],
}



import os
from typing import Dict
import requests
from PySide6.QtCore import Signal
from airunner.enums import SignalCode
from airunner.workers.worker import Worker
from airunner.settings import DEFAULT_HF_ENDPOINT


class DownloadWorker(Worker):
    progress = Signal(int, int)  # current, total
    finished = Signal(dict)
    failed = Signal(Exception)
    running = False
    is_cancelled = False

    @staticmethod
    def get_size(url: str):
        try:
            response = requests.head(url, allow_redirects=True)
            size_kb = int(response.headers.get("content-length", 0))
            return size_kb
        except OverflowError:
            print(f"OverflowError when getting size for {url}")
            raise

    def cancel(self):
        self.is_cancelled = True

    def handle_message(self, data: Dict):
        self.running = True
        path = data.get("requested_path", "")
        file_name = data.get("requested_file_name", "")
        file_path = data.get("requested_file_path", "")
        callback = data.get("requested_callback", None)

        if path == "" and file_name == "" and file_path == "":
            callback()
            return
        url = f"{DEFAULT_HF_ENDPOINT}/{path}/resolve/main/{file_name}?download=true".replace(
            " ", ""
        )
        self.emit_signal(SignalCode.CLEAR_DOWNLOAD_STATUS_BAR)
        self.emit_signal(
            SignalCode.SET_DOWNLOAD_STATUS_LABEL,
            {"message": f"Downloading {file_name}"},
        )

        file_name = os.path.join(file_path, file_name)
        file_name = os.path.expanduser(file_name)
        if not os.path.exists(os.path.dirname(file_name)):
            try:
                os.makedirs(os.path.dirname(file_name), exist_ok=True)
            except FileExistsError:
                pass

        if os.path.exists(file_name):
            self.emit_signal(
                SignalCode.UPDATE_DOWNLOAD_LOG,
                {"message": f"File already exists, skipping download"},
            )
            self.emit_signal(
                SignalCode.DOWNLOAD_PROGRESS, {"current": 0, "total": 0}
            )
            self.api.download_complete(file_name)
            return

        size_kb = self.get_size(url)
        self.emit_signal(
            SignalCode.UPDATE_DOWNLOAD_LOG,
            {
                "message": f"Downloading {url} of size {size_kb} KB to {file_name}"
            },
        )

        try:
            headers = {}
            with requests.get(
                url, headers=headers, stream=True, allow_redirects=True
            ) as r:
                r.raise_for_status()
                dir_name = os.path.dirname(file_name)

                if dir_name != "" and not os.path.exists(dir_name):
                    try:
                        os.makedirs(dir_name, exist_ok=True)
                    except FileExistsError:
                        pass

                with open(file_name, "wb") as f:
                    for chunk in r.iter_content(chunk_size=8192):
                        if self.is_cancelled:
                            break
                        try:
                            f.write(chunk)
                        except OverflowError:
                            print(f"OverflowError when writing to {file_name}")
                            raise
                        self.emit_signal(
                            SignalCode.DOWNLOAD_PROGRESS,
                            {"current": f.tell(), "total": size_kb},
                        )
                    self.emit_signal(
                        SignalCode.UPDATE_DOWNLOAD_LOG,
                        {"message": f"finished with download of {file_name}"},
                    )
                    # Also emit the DOWNLOAD_COMPLETE signal to update the progress bar
                    self.api.download_complete(file_name)
                    self.finished.emit({})
        except Exception as e:
            print(f"Failed to download {url}")
            print(e)
            self.failed.emit(e)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'brush_widget.ui'
##
## Created by: Qt User Interface Compiler version 6.9.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QFrame, QGridLayout,
    QGroupBox, QHBoxLayout, QLabel, QScrollArea,
    QSizePolicy, QSpacerItem, QToolButton, QWidget)

from airunner.gui.widgets.slider.slider_widget import SliderWidget

class Ui_brush_widget(object):
    def setupUi(self, brush_widget):
        if not brush_widget.objectName():
            brush_widget.setObjectName(u"brush_widget")
        brush_widget.resize(1004, 622)
        self.gridLayout_2 = QGridLayout(brush_widget)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setHorizontalSpacing(0)
        self.gridLayout_2.setVerticalSpacing(10)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.scrollArea = QScrollArea(brush_widget)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 1002, 580))
        self.gridLayout = QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setHorizontalSpacing(0)
        self.gridLayout.setVerticalSpacing(10)
        self.gridLayout.setContentsMargins(0, 0, 10, 0)
        self.widget_2 = QWidget(self.scrollAreaWidgetContents)
        self.widget_2.setObjectName(u"widget_2")
        self.widget_2.setMinimumSize(QSize(0, 0))
        self.widget_2.setStyleSheet(u"")
        self.horizontalLayout_2 = QHBoxLayout(self.widget_2)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, -1)
        self.brush_size_slider = SliderWidget(self.widget_2)
        self.brush_size_slider.setObjectName(u"brush_size_slider")
        self.brush_size_slider.setProperty(u"slider_minimum", 1)
        self.brush_size_slider.setProperty(u"slider_maximum", 100)
        self.brush_size_slider.setProperty(u"spinbox_minimum", 1)
        self.brush_size_slider.setProperty(u"spinbox_maximum", 100)
        self.brush_size_slider.setProperty(u"slider_tick_interval", 1)
        self.brush_size_slider.setProperty(u"slider_single_step", 1)
        self.brush_size_slider.setProperty(u"slider_page_step", 10)
        self.brush_size_slider.setProperty(u"spinbox_single_step", 1)
        self.brush_size_slider.setProperty(u"spinbox_page_step", 10)
        self.brush_size_slider.setProperty(u"settings_property", u"brush_settings.size")
        self.brush_size_slider.setProperty(u"display_as_float", False)

        self.horizontalLayout_2.addWidget(self.brush_size_slider)


        self.gridLayout.addWidget(self.widget_2, 0, 0, 1, 1)

        self.toggle_auto_generate_while_drawing = QCheckBox(self.scrollAreaWidgetContents)
        self.toggle_auto_generate_while_drawing.setObjectName(u"toggle_auto_generate_while_drawing")

        self.gridLayout.addWidget(self.toggle_auto_generate_while_drawing, 3, 0, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 4, 0, 1, 1)

        self.groupBox_4 = QGroupBox(self.scrollAreaWidgetContents)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.groupBox_4.setMinimumSize(QSize(0, 0))
        font = QFont()
        font.setPointSize(10)
        font.setBold(True)
        self.groupBox_4.setFont(font)
        self.gridLayout_3 = QGridLayout(self.groupBox_4)
        self.gridLayout_3.setSpacing(0)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.primary_color_button = QToolButton(self.groupBox_4)
        self.primary_color_button.setObjectName(u"primary_color_button")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.primary_color_button.sizePolicy().hasHeightForWidth())
        self.primary_color_button.setSizePolicy(sizePolicy)
        self.primary_color_button.setFont(font)

        self.gridLayout_3.addWidget(self.primary_color_button, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.groupBox_4, 2, 0, 1, 1)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.gridLayout_2.addWidget(self.scrollArea, 2, 0, 1, 1)

        self.label = QLabel(brush_widget)
        self.label.setObjectName(u"label")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Minimum)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy1)
        font1 = QFont()
        font1.setPointSize(11)
        font1.setBold(True)
        self.label.setFont(font1)

        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 1)

        self.line = QFrame(brush_widget)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout_2.addWidget(self.line, 1, 0, 1, 1)


        self.retranslateUi(brush_widget)
        self.primary_color_button.clicked.connect(brush_widget.color_button_clicked)
        self.toggle_auto_generate_while_drawing.toggled.connect(brush_widget.toggle_auto_generate_while_drawing)

        QMetaObject.connectSlotsByName(brush_widget)
    # setupUi

    def retranslateUi(self, brush_widget):
        brush_widget.setWindowTitle(QCoreApplication.translate("brush_widget", u"Form", None))
        self.brush_size_slider.setProperty(u"label_text", QCoreApplication.translate("brush_widget", u"Brush Size", None))
        self.toggle_auto_generate_while_drawing.setText(QCoreApplication.translate("brush_widget", u"Auto-generate while drawing", None))
        self.groupBox_4.setTitle(QCoreApplication.translate("brush_widget", u"Brush color", None))
        self.primary_color_button.setText("")
        self.label.setText(QCoreApplication.translate("brush_widget", u"Brush Settings", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'stable_diffusion_welcome_screen.ui'
##
## Created by: Qt User Interface Compiler version 6.9.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QGridLayout, QLabel,
    QRadioButton, QSizePolicy, QSpacerItem, QWidget)

class Ui_stable_diffusion_welcome_screen(object):
    def setupUi(self, stable_diffusion_welcome_screen):
        if not stable_diffusion_welcome_screen.objectName():
            stable_diffusion_welcome_screen.setObjectName(u"stable_diffusion_welcome_screen")
        stable_diffusion_welcome_screen.resize(403, 251)
        self.gridLayout = QGridLayout(stable_diffusion_welcome_screen)
        self.gridLayout.setObjectName(u"gridLayout")
        self.verticalSpacer = QSpacerItem(20, 139, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 6, 0, 1, 1)

        self.line = QFrame(stable_diffusion_welcome_screen)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line, 2, 0, 1, 1)

        self.label = QLabel(stable_diffusion_welcome_screen)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        self.label.setFont(font)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.radioButton = QRadioButton(stable_diffusion_welcome_screen)
        self.radioButton.setObjectName(u"radioButton")
        self.radioButton.setChecked(True)

        self.gridLayout.addWidget(self.radioButton, 4, 0, 1, 1)

        self.radioButton_2 = QRadioButton(stable_diffusion_welcome_screen)
        self.radioButton_2.setObjectName(u"radioButton_2")

        self.gridLayout.addWidget(self.radioButton_2, 5, 0, 1, 1)

        self.label_2 = QLabel(stable_diffusion_welcome_screen)
        self.label_2.setObjectName(u"label_2")
        font1 = QFont()
        font1.setBold(True)
        self.label_2.setFont(font1)

        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)

        self.label_3 = QLabel(stable_diffusion_welcome_screen)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setWordWrap(True)

        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)


        self.retranslateUi(stable_diffusion_welcome_screen)
        self.radioButton.toggled.connect(stable_diffusion_welcome_screen.yes_toggled)
        self.radioButton_2.toggled.connect(stable_diffusion_welcome_screen.no_toggled)

        QMetaObject.connectSlotsByName(stable_diffusion_welcome_screen)
    # setupUi

    def retranslateUi(self, stable_diffusion_welcome_screen):
        stable_diffusion_welcome_screen.setWindowTitle(QCoreApplication.translate("stable_diffusion_welcome_screen", u"Form", None))
        self.label.setText(QCoreApplication.translate("stable_diffusion_welcome_screen", u"Stable Diffusion setup", None))
        self.radioButton.setText(QCoreApplication.translate("stable_diffusion_welcome_screen", u"Yes", None))
        self.radioButton_2.setText(QCoreApplication.translate("stable_diffusion_welcome_screen", u"No", None))
        self.label_2.setText(QCoreApplication.translate("stable_diffusion_welcome_screen", u"Would you like to use Stable Diffusion as your Image Generator?", None))
        self.label_3.setText(QCoreApplication.translate("stable_diffusion_welcome_screen", u"Stable Diffusion is an open-source AI image generator powered by Diffusers. It enables the creation of images from text prompts. Depending on the selected model, there is a potential for generating content that could be considered unsafe or disturbing. While measures have been implemented to mitigate this risk, no AI model can be guaranteed to be completely safe.", None))
    # retranslateUi




from sqlalchemy import Column, Integer, String, Float, ForeignKey, JSON, Text
from sqlalchemy.orm import relationship
from airunner.data.models.base import BaseModel
from airunner.data.models.base_manager import BaseManager


class WorkflowNode(BaseModel):
    __tablename__ = "workflow_nodes"
    NODE_NAME = "Workflow Node"

    id = Column(Integer, primary_key=True)
    workflow_id = Column(Integer, ForeignKey("workflows.id"), nullable=False)
    node_identifier = Column(
        String, nullable=False
    )  # e.g., 'ai_runner.nodes.AgentActionNode'
    name = Column(String, nullable=False)
    pos_x = Column(Float, default=0.0)
    pos_y = Column(Float, default=0.0)
    properties = Column(
        JSON, default=lambda: {}
    )  # Store node-specific settings, including port definitions

    workflow = relationship("Workflow", back_populates="nodes")

    # Relationships for connections (useful for cascading deletes)
    output_connections = relationship(
        "WorkflowConnection",
        foreign_keys="[WorkflowConnection.output_node_id]",
        back_populates="output_node",
        cascade="all, delete-orphan",
    )
    input_connections = relationship(
        "WorkflowConnection",
        foreign_keys="[WorkflowConnection.input_node_id]",
        back_populates="input_node",
        cascade="all, delete-orphan",
    )

    # Initialize the manager after the class is fully defined
    objects = None

    def __repr__(self):
        return f"<WorkflowNode(id={self.id}, name='{self.name}', identifier='{self.node_identifier}')>"


# Set the manager outside the class definition
WorkflowNode.objects = BaseManager(WorkflowNode)



from sqlalchemy import Column, Integer, String

from airunner.data.models.base import BaseModel


class EspeakSettings(BaseModel):
    __tablename__ = "espeak_settings"
    id = Column(Integer, primary_key=True, autoincrement=True)
    gender = Column(String, default="Male")
    voice = Column(String, default="english (america)")
    language = Column(String, default="en-US")
    rate = Column(Integer, default=100)
    pitch = Column(Integer, default=100)
    volume = Column(Integer, default=100)
    punctuation_mode = Column(String, default="none")



from typing import Any
import queue
from abc import abstractmethod, ABC, ABCMeta

from PySide6.QtCore import Signal, QThread, QObject

from airunner.enums import QueueType, SignalCode, WorkerState
from airunner.utils.application.mediator_mixin import MediatorMixin
from airunner.settings import AIRUNNER_SLEEP_TIME_IN_MS
from airunner.gui.windows.main.settings_mixin import SettingsMixin
from airunner.api import API


QObjectMeta = type(QObject)


class CombinedMeta(QObjectMeta, ABCMeta):
    pass


class Worker(
    MediatorMixin, SettingsMixin, QObject, ABC, metaclass=CombinedMeta
):
    queue_type = QueueType.GET_NEXT_ITEM
    finished = Signal()
    prefix = "Worker"

    def __init__(
        self,
        sleep_time_in_ms: int = AIRUNNER_SLEEP_TIME_IN_MS,
        initialize_gui=True,
    ):
        super().__init__()
        self._sleep_time_in_ms: int = sleep_time_in_ms
        self.state = WorkerState.HALTED
        self.running = False
        self.queue = queue.Queue()
        self.items = {}
        self.current_index = 0
        self.paused = False
        self.api = API(initialize_gui=initialize_gui)
        self.register(SignalCode.QUIT_APPLICATION, self.stop)

        # Remove thread creation and management from here
        # as it's handled by create_worker function

    @abstractmethod
    def handle_message(self, message: Any):
        raise NotImplementedError

    def start_worker_thread(self):
        """
        This method is kept for backward compatibility
        but actual thread management is handled by create_worker
        """
        pass

    def start(self):
        """Start the worker's processing loop"""
        self.run()

    def run(self):
        if self.queue_type is QueueType.NONE:
            return
        self.logger.debug("Starting worker")
        self.running = True
        while self.running:
            self.preprocess()
            self.run_thread()

    def run_thread(self):
        """
        For most workers you can override
        the handle_message method and leave this
        default run_thread implmentation in place.
        """
        try:
            msg = self.get_item_from_queue()
            if msg is not None:
                self.handle_message(msg)
        except queue.Empty:
            pass
        if self.paused:
            self.logger.debug("Paused")
            while self.paused:
                QThread.msleep(self._sleep_time_in_ms)
            self.logger.debug("Resumed")
        QThread.msleep(self._sleep_time_in_ms)

    def preprocess(self):
        pass

    def clear_queue(self):
        self.queue = queue.Queue()

    def get_item_from_queue(self):
        if self.queue_type == QueueType.GET_LAST_ITEM:
            msg = self.get_last_item()
        else:
            msg = self.get_next_item()
        return msg

    def get_last_item(self):
        index = None
        while not self.queue.empty():
            try:
                index = self.queue.get_nowait()
                if index in self.items:
                    break
            except queue.Empty:
                pass
        if index in self.items:
            msg = self.items.pop(index)
            self.items = {}
            self.queue.empty()
            return msg
        else:
            return None

    def get_next_item(self):
        try:
            index = self.queue.get_nowait()
            msg = self.items.pop(index, None)
            return msg
        except queue.Empty:
            return None

    def pause(self):
        self.state = WorkerState.PAUSED

    def unpause(self):
        if self.state == WorkerState.PAUSED:
            self.state = WorkerState.RUNNING

    def resume(self):
        self.paused = False

    def add_to_queue(self, message):
        if (
            type(message) is dict
            and "options" in message
            and message["options"]["empty_queue"]
        ):
            self.empty_queue()

        if self.queue_type == QueueType.GET_LAST_ITEM:
            self.items = {}
            self.queue.empty()
            self.current_index = 0

        self.items[self.current_index] = message
        self.queue.put(self.current_index)
        self.current_index += 1

    def empty_queue(self):
        self.queue = queue.Queue()
        self.items = {}
        self.current_index = 0

    def stop(self):
        self.logger.debug("Stopping")
        self.running = False
        self.finished.emit()

    def cancel(self):
        self.logger.debug("Canceling")
        while not self.queue.empty():
            self.queue.get()



from airunner.utils.memory.clear_memory import clear_memory
from airunner.utils.memory.gpu_memory_stats import gpu_memory_stats
from airunner.utils.memory.is_ampere_or_newer import is_ampere_or_newer


__all__ = [
    "clear_memory",
    "gpu_memory_stats",
    "is_ampere_or_newer",
]



import os

from pathlib import Path

from logging.config import fileConfig
from sqlalchemy import engine_from_config, pool
from alembic import context
from airunner.settings import AIRUNNER_DB_URL

config = context.config
config.set_main_option("sqlalchemy.url", AIRUNNER_DB_URL)

# Check if DB file exists
if AIRUNNER_DB_URL.__contains__("sqlite"):
    db_path = AIRUNNER_DB_URL.replace("sqlite:///", "")
    try:
        if not os.path.exists(db_path):
            print(f"Database file not found at {db_path}")
            # create the file
            os.makedirs(os.path.dirname(db_path), exist_ok=True)
            with open(db_path, "w") as f:
                pass
            print(f"Database file created at {db_path}")
    except Exception as e:
        print(f"Error checking DB path: {e}")

# Import your models here
from airunner.data.models.base import Base

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Get the path to the alembic.ini file
config_file_path = Path(__file__).parent / "../alembic.ini"

# Set the config file name explicitly
config.config_file_name = str(config_file_path)

# Interpret the config file for Python logging
fileConfig(config.config_file_name)

target_metadata = Base.metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


def run_migrations_offline():
    """Run migrations in 'offline' mode.
    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well. By skipping the Engine creation
    we don't even need a DBAPI to be available.
    Calls to context.execute() here emit the given string to the
    script output.
    """
    url = config.get_main_option("sqlalchemy.url")
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
        compare_type=False,
        compare_server_default=False,
    )

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    """Run migrations in 'online' mode.
    In this scenario we need to create an Engine
    and associate a connection with the context.
    """
    connectable = engine_from_config(
        config.get_section(config.config_ini_section),
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=target_metadata,
            compare_type=False,
            compare_server_default=False,
        )

        with context.begin_transaction():
            context.run_migrations()
            connection.commit()
    print("Migrations completed successfully.")


if context.is_offline_mode():
    print("Running migrations offline...")
    run_migrations_offline()
else:
    print("Running migrations online...")
    run_migrations_online()



from typing import Callable
from PySide6.QtCore import QObject, Signal
from airunner.workers.download_worker import DownloadWorker
from airunner.enums import SignalCode
from airunner.utils.application.mediator_mixin import MediatorMixin
from airunner.gui.windows.main.settings_mixin import SettingsMixin
from airunner.utils.application.create_worker import create_worker


class HuggingfaceDownloader(
    MediatorMixin,
    SettingsMixin,
    QObject,
):
    completed = Signal()

    def __init__(self, callback=None, initialize_gui=True):
        super().__init__()
        self.thread = None
        self.worker = None
        self.downloading = False

        self.worker = create_worker(
            DownloadWorker, initialize_gui=initialize_gui
        )

        # Connect signals
        self.worker.progress.connect(
            lambda current, total: callback(current, total)
        )

        self.logger.debug(f"Starting model download thread")
        self.worker.finished.connect(self.handle_completed)

    def download_model(
        self,
        requested_path: str,
        requested_file_name: str,
        requested_file_path: str,
        requested_callback: Callable[[int, int], None],
    ):
        self.worker.add_to_queue(
            dict(
                requested_path=requested_path,
                requested_file_name=requested_file_name,
                requested_file_path=requested_file_path,
                requested_callback=requested_callback,
            )
        )

    def handle_completed(self):
        self.completed.emit()

    def stop_download(self):
        if self.worker:
            self.worker.cancel()



from PySide6.QtGui import QTransform

from airunner.utils.application.mediator_mixin import MediatorMixin
from airunner.gui.windows.main.settings_mixin import SettingsMixin


class ZoomHandler(
    MediatorMixin,
    SettingsMixin,
):
    @property
    def zoom_in_step(self) -> float:
        zoom_level = self.grid_settings.zoom_level
        if zoom_level > 6:
            return 2
        elif zoom_level > 4:
            return 1
        return self.grid_settings.zoom_in_step

    @property
    def zoom_out_step(self) -> float:
        zoom_level = self.grid_settings.zoom_level
        if zoom_level > 6:
            return 2
        elif zoom_level > 4:
            return 1
        if zoom_level <= 1.0:
            return 0.05
        return self.grid_settings.zoom_out_step

    @property
    def zoom_level(self) -> float:
        zoom = self.grid_settings.zoom_level
        if zoom <= 0:
            zoom = 0.1
        return zoom

    @zoom_level.setter
    def zoom_level(self, value):
        self.update_grid_settings("zoom_level", value)

    def on_zoom_level_changed(self) -> QTransform:
        # Create a QTransform object and scale it
        zoom_level = self.grid_settings.zoom_level
        transform = QTransform()
        transform.scale(zoom_level, zoom_level)
        return transform



import os
import time
from queue import Queue
import requests
from PySide6.QtCore import QObject, Signal
from airunner.enums import SignalCode
from airunner.utils.application.mediator_mixin import MediatorMixin
from airunner.gui.windows.main.settings_mixin import SettingsMixin


class CivitAIDownloadWorker(MediatorMixin, SettingsMixin, QObject):
    """
    Worker class for downloading files from CivitAI with progress tracking and cancellation support.
    """

    progress = Signal(int, int)  # current, total
    finished = Signal()
    failed = Signal(Exception)

    def __init__(self, *args, **kwargs):
        super().__init__()
        self.queue = Queue()
        self.running = False
        self.is_cancelled = False

    def add_to_queue(self, data: tuple):
        """Add a download task to the queue."""
        self.queue.put(data)

    @staticmethod
    def get_size(url: str) -> int:
        """Get the size of the file at the given URL in kilobytes."""
        try:
            response = requests.head(url, allow_redirects=True)
            return int(response.headers.get("content-length", 0))
        except OverflowError:
            raise OverflowError(f"OverflowError when getting size for {url}")

    def cancel(self):
        """Cancel the current download process."""
        self.is_cancelled = True

    def download(self):
        """Process the download queue and handle file downloads."""
        self.running = True
        while self.running:
            if self.queue.empty():
                time.sleep(0.1)
                continue

            url, file_name, size_kb = self.queue.get()
            size_kb *= 1024  # Convert size from KB to bytes

            self.api.clear_download_status()
            self.api.set_download_status(f"Downloading {file_name}")

            file_name = os.path.expanduser(file_name)
            try:
                os.makedirs(file_name, exist_ok=True)
            except FileExistsError:
                pass

            self.api.update_download_log(
                f"Downloading {url} of size {size_kb} bytes to {file_name}"
            )

            if os.path.exists(file_name):
                self.api.update_download_log(
                    "File already exists, skipping download"
                )
                self.api.set_download_progress(current=size_kb, total=size_kb)
                self.progress.emit(size_kb, size_kb)
                self.finished.emit()
                continue

            try:
                with requests.get(url, stream=True, allow_redirects=True) as r:
                    r.raise_for_status()
                    with open(file_name, "wb") as f:
                        for chunk in r.iter_content(chunk_size=8192):
                            if self.is_cancelled:
                                break
                            f.write(chunk)
                            self.api.set_download_progress(
                                current=f.tell(), total=size_kb
                            )
                            self.progress.emit(f.tell(), size_kb)

                self.api.update_download_log(
                    f"Finished downloading {file_name}"
                )
                self.finished.emit()

            except Exception as e:
                self.failed.emit(e)
                self.api.download_complete(file_name)



import os
from PySide6.QtWidgets import QFileDialog


def show_path(path):
    path = os.path.expanduser(path)
    if not os.path.isdir(path):
        print(f"Error: {path} is not a valid directory")  # Debug statement
        return

    # Use QFileDialog to open a file browser dialog
    file_dialog = QFileDialog()
    file_dialog.setFileMode(QFileDialog.Directory)
    file_dialog.setDirectory(path)
    # file_dialog.setOption(QFileDialog.ShowDirsOnly, True)

    if file_dialog.exec():
        selected_directory = file_dialog.selectedFiles()[0]
    else:
        print("User canceled the file dialog")  # Debug statement



import math


def snap_to_grid(
    settings, x: float, y: float, use_floor: bool = True
):  # Accept floats
    cell_size = float(
        settings.cell_size
    )  # Ensure cell_size is float for division
    if settings.snap_to_grid and cell_size > 0:  # Add check for cell_size > 0
        if use_floor:
            x = math.floor(x / cell_size) * cell_size
            y = math.floor(y / cell_size) * cell_size
        else:
            # Round to the nearest grid point
            x = round(x / cell_size) * cell_size
            y = round(y / cell_size) * cell_size
    # Return floats for potentially more precise positioning if needed downstream
    return float(x), float(y)



from PySide6.QtGui import QFontDatabase, QFont
from PySide6.QtWidgets import QTextEdit, QApplication, QWidget, QVBoxLayout
from PySide6.QtGui import QFontMetrics
from PySide6.QtCore import Qt, QSize, Slot, QEvent, QTimer, QThread, QObject
from PySide6.QtCore import Signal, QPropertyAnimation, QEasingCurve
import queue

from airunner.enums import SignalCode
from airunner.gui.widgets.base_widget import BaseWidget
from airunner.gui.widgets.llm.templates.message_ui import Ui_message
from airunner.data.models import Conversation
from airunner.data.session_manager import session_scope
from airunner.gui.widgets.llm.content_widgets import (
    PlainTextWidget,
    LatexWidget,
    MarkdownWidget,
    MixedContentWidget,
)
from airunner.utils.text.formatter_extended import FormatterExtended


class AutoResizingTextEdit(QTextEdit):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.document().contentsChanged.connect(self.sizeChange)


# Worker class for the resize operation
class ResizeWorker(QObject):
    finished = Signal()

    def __init__(self, message_queue):
        super().__init__()
        self.message_queue = message_queue
        self.running = True

    def process(self):
        while self.running:
            try:
                # Get the next message widget from the queue with a timeout
                message_widget = self.message_queue.get(timeout=0.1)
                message_widget.set_content_size()
                self.message_queue.task_done()
            except queue.Empty:
                # If the queue is empty, just continue and check again
                pass

    def stop(self):
        self.running = False


class MessageWidget(BaseWidget):
    widget_class_ = Ui_message
    textChanged = Signal()
    icons = [
        ("copy", "copy_button"),
        ("x-circle", "delete_button"),
        ("play", "play_audio_button"),
    ]

    # Class-level thread and queue for all instances
    resize_queue = queue.Queue()
    resize_thread = None
    resize_worker = None

    @classmethod
    def initialize_resize_worker(cls):
        if cls.resize_thread is None:
            cls.resize_thread = QThread()
            cls.resize_worker = ResizeWorker(cls.resize_queue)
            cls.resize_worker.moveToThread(cls.resize_thread)

            # Connect signals and slots
            cls.resize_thread.started.connect(cls.resize_worker.process)

            # Start the thread
            cls.resize_thread.start()

    def __init__(self, *args, **kwargs):
        self.signal_handlers = {
            SignalCode.APPLICATION_SETTINGS_CHANGED_SIGNAL: self.on_application_settings_changed_signal,
            SignalCode.DELETE_MESSAGES_AFTER_ID: self.on_delete_messages_after_id,
        }
        self.name = kwargs.pop("name")
        self.message = kwargs.pop("message")
        self.message_id = kwargs.pop("message_id")
        self.conversation_id = kwargs.pop("conversation_id")
        self.is_bot = kwargs.pop("is_bot")
        super().__init__(*args, **kwargs)

        # Initialize the class-level worker if not already done
        self.__class__.initialize_resize_worker()

        self._deleted = False
        self.ui.user_name.setText(f"{self.name}")
        self.font_family = None
        self.font_size = None

        # Set up content container layout
        self.content_layout = QVBoxLayout(self.ui.content_container)
        self.content_layout.setContentsMargins(0, 0, 0, 0)
        self.content_layout.setSpacing(0)

        # Initialize content widget to None
        self.content_widget = None

        # Set the chat font
        self.set_chat_font()

        if self.is_bot:
            self.ui.message_container.setProperty("class", "alternate")

        # Set size policies for better expansion
        from PySide6.QtWidgets import QSizePolicy

        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.ui.content_container.setSizePolicy(
            QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        # Always keep buttons in the layout but make them transparent when not visible
        # This prevents layout shifts when hovering
        self.ui.copy_button.setVisible(True)
        self.ui.delete_button.setVisible(True)
        self.ui.play_audio_button.setVisible(True)

        # Set opacity to make them invisible but keep their layout space
        self.ui.copy_button.setStyleSheet(
            "background: transparent; border: none;"
        )
        self.ui.delete_button.setStyleSheet(
            "background: transparent; border: none;"
        )
        self.ui.play_audio_button.setStyleSheet(
            "background: transparent; border: none;"
        )

        # Create opacity effects for smooth transitions
        from PySide6.QtWidgets import QGraphicsOpacityEffect

        # Create opacity effects for each button
        self.copy_opacity = QGraphicsOpacityEffect()
        self.delete_opacity = QGraphicsOpacityEffect()
        self.play_opacity = QGraphicsOpacityEffect()

        # Set initial opacity to 0.05 (almost invisible but still clickable)
        self.copy_opacity.setOpacity(0.05)
        self.delete_opacity.setOpacity(0.05)
        self.play_opacity.setOpacity(0.05)

        # Apply effects to buttons
        self.ui.copy_button.setGraphicsEffect(self.copy_opacity)
        self.ui.delete_button.setGraphicsEffect(self.delete_opacity)
        self.ui.play_audio_button.setGraphicsEffect(self.play_opacity)

        # Create animations
        self.copy_anim = QPropertyAnimation(self.copy_opacity, b"opacity")
        self.delete_anim = QPropertyAnimation(self.delete_opacity, b"opacity")
        self.play_anim = QPropertyAnimation(self.play_opacity, b"opacity")

        # Configure animations
        for anim in [self.copy_anim, self.delete_anim, self.play_anim]:
            anim.setDuration(150)  # 150ms duration for smooth transition
            anim.setEasingCurve(QEasingCurve.Type.InOutQuad)

        self.ui.message_container.installEventFilter(self)
        self.set_cursor(Qt.CursorShape.ArrowCursor)

        # Set a global tooltip style for readability
        QApplication.instance().setStyleSheet(
            QApplication.instance().styleSheet()
            + """
        QToolTip {
            color: #fff;
            background-color: #222;
            border: 1px solid #555;
            padding: 4px 8px;
            font-size: 13px;
            border-radius: 4px;
        }
        """
        )

        # Add a stylesheet for action buttons for hover/pressed feedback
        button_style = """
        QPushButton {
            background: transparent;
            border: none;
            border-radius: 3px;
        }
        QPushButton:hover {
            background: #333;
            border: 1px solid #666;
        }
        QPushButton:pressed {
            background: #444;
            border: 1px solid #888;
        }
        """
        self.ui.copy_button.setStyleSheet(button_style)
        self.ui.delete_button.setStyleSheet(button_style)
        self.ui.play_audio_button.setStyleSheet(button_style)

        # Set the cursor to pointing hand for action buttons
        self.ui.copy_button.setCursor(Qt.PointingHandCursor)
        self.ui.delete_button.setCursor(Qt.PointingHandCursor)
        self.ui.play_audio_button.setCursor(Qt.PointingHandCursor)

        # Remove tooltips from action buttons
        self.ui.copy_button.setToolTip("")
        self.ui.delete_button.setToolTip("")
        self.ui.play_audio_button.setToolTip("")

        # Hide image_content by default
        self.ui.image_content.setVisible(False)

        # Set message content
        self.set_message_content(self.message)

    def set_message_content(self, message):
        """
        Set the message content in the widget, displaying the appropriate specialized widget
        based on content type.
        """
        # Clear any existing content widget
        if self.content_widget is not None:
            self.content_layout.removeWidget(self.content_widget)
            self.content_widget.deleteLater()
            self.content_widget = None

        # Use the extended formatter to get detailed content information
        result = FormatterExtended.format_content(message)

        # Create and configure the appropriate content widget based on content type
        if result["type"] == FormatterExtended.FORMAT_MIXED:
            self.content_widget = MixedContentWidget(self.ui.content_container)
            self.content_widget.setContent(result["parts"])

        elif result["type"] == FormatterExtended.FORMAT_LATEX:
            self.content_widget = LatexWidget(self.ui.content_container)
            self.content_widget.setContent(result["content"])

        elif result["type"] == FormatterExtended.FORMAT_MARKDOWN:
            self.content_widget = MarkdownWidget(self.ui.content_container)
            self.content_widget.setContent(result["content"])

        else:  # Plain text (default)
            self.content_widget = PlainTextWidget(self.ui.content_container)
            self.content_widget.setContent(result["content"])

        # Apply font settings to the content widget
        if self.font_family and self.font_size:
            font = QFont(self.font_family, self.font_size)
            self.content_widget.setFont(font)

        # Add the content widget to the container
        self.content_layout.addWidget(self.content_widget)

        # Connect size changed signal
        if hasattr(self.content_widget, "sizeChanged"):
            self.content_widget.sizeChanged.connect(self.content_size_changed)

    def content_size_changed(self):
        """Handle content widget size changes."""
        # Update our own size hints based on the content widget
        if self.content_widget:
            self.updateGeometry()

    def on_delete_messages_after_id(self, data):
        message_id = data.get("message_id", None)
        if self.message_id > message_id:
            if (
                not self._deleted
            ):  # Check if the widget has already been deleted
                try:
                    if (
                        self.parentWidget()
                    ):  # Check if the widget still has a parent
                        self.deleteLater()
                except RuntimeError:
                    pass

    def set_cursor(self, cursor_type):
        self.ui.message_container.setCursor(cursor_type)
        for child in self.ui.message_container.findChildren(QWidget):
            child.setCursor(cursor_type)

    def eventFilter(self, obj, event):
        if obj == self.ui.message_container:
            if event.type() == QEvent.Type.Enter:
                # Show buttons with smooth animation
                for anim in [self.copy_anim, self.delete_anim, self.play_anim]:
                    anim.stop()
                    anim.setStartValue(
                        anim.currentValue()
                        if anim.state() == QPropertyAnimation.State.Running
                        else 0.05
                    )
                    anim.setEndValue(1.0)
                    anim.start()
            elif event.type() == QEvent.Type.Leave:
                # Hide buttons with smooth animation (to 0.05, not 0)
                for anim in [self.copy_anim, self.delete_anim, self.play_anim]:
                    anim.stop()
                    anim.setStartValue(
                        anim.currentValue()
                        if anim.state() == QPropertyAnimation.State.Running
                        else 1.0
                    )
                    anim.setEndValue(0.05)
                    anim.start()
        return super().eventFilter(obj, event)

    def on_application_settings_changed_signal(self):
        self.set_chat_font()

    def set_chat_font(self):
        font_setting = self.get_font_setting_by_name("chat")
        font_family = font_setting.font_family
        font_size = font_setting.font_size
        if self.font_family != font_family or self.font_size != font_size:
            self.font_family = font_family
            self.font_size = font_size
            # Check if the font family is available
            available_families = QFontDatabase().families()
            if self.font_family in available_families:
                font = QFont(self.font_family, self.font_size)
            else:
                font = QFont(
                    "Ubuntu", self.font_size
                )  # Fallback to Ubuntu if Arial is not available
            font.setFamilies(
                [
                    font.family(),
                    "Noto Color Emoji",
                ]
            )

            # Apply the font to the content widget if it exists
            if self.content_widget:
                self.content_widget.setFont(font)

    def set_content_size(self):
        # Let the content widget handle its own sizing
        pass

    def sizeChange(self):
        # Notify that the text has changed
        self.textChanged.emit()

    def resizeEvent(self, event):
        super().resizeEvent(event)

    def sizeHint(self):
        # Use the content widget's size hint if available
        if self.content_widget:
            content_size = self.content_widget.sizeHint()
            return QSize(
                content_size.width(), content_size.height() + 60
            )  # Add space for header

        # Fallback sizing logic
        lines = self.message.count("\n") + 1
        chars_per_line = (
            max(len(line) for line in self.message.split("\n") if line)
            if self.message
            else 30
        )

        # Estimate size based on content
        width = min(max(chars_per_line * 10, 300), 1000)  # Reasonable width
        height = max(100, min(lines * 24, 800))  # Reasonable height per line

        return QSize(width, height + 50)  # Add space for header

    def minimumSizeHint(self):
        # Use the content widget's minimum size hint if available
        if self.content_widget:
            content_min_size = self.content_widget.minimumSizeHint()
            return QSize(
                content_min_size.width(), content_min_size.height() + 60
            )  # Add space for header

        # Reasonable minimum size
        return QSize(300, 100)

    def update_message(self, text):
        self.message += text
        # strip double spaces from self.message
        self.message = self.message.replace("  ", " ")

        # Update the content
        self.set_message_content(self.message)

    @Slot()
    def on_play_audio_button_clicked(self):
        self.api.tts.play_audio(self.message)

    @Slot()
    def delete(self):
        if not self._deleted:  # Check if the widget has already been deleted
            self._deleted = True
            with session_scope() as session:
                conversation = (
                    session.query(Conversation)
                    .filter(Conversation.id == self.conversation_id)
                    .first()
                )
                messages = conversation.value
                if self.message_id == 0:
                    conversation.value = []
                else:
                    conversation.value = messages[0 : self.message_id]
                session.add(conversation)
                session.commit()
            self.api.llm.delete_messages_after_id(self.message_id)
            self.setParent(None)
            QTimer.singleShot(0, self.deleteLater)

    @Slot()
    def copy(self):
        clipboard = QApplication.clipboard()
        clipboard.setText(self.message)



from typing import Annotated
from dataclasses import dataclass
from airunner.enums import AvailableLanguage


@dataclass
class TTSRequest:
    message: Annotated[str, "The message to send to the text-to-speech engine"]
    gender: Annotated[str, "The gender of the voice"] = "Male"
    language: Annotated[AvailableLanguage, "Language of the voice"] = (
        AvailableLanguage.EN
    )


@dataclass
class EspeakTTSRequest(TTSRequest):
    rate: Annotated[int, "Rate of speech in words per minute"] = 100
    pitch: Annotated[int, "Pitch of the voice"] = 100
    volume: Annotated[int, "Volume level"] = 100
    voice: Annotated[str, "Voice to use for speech"] = "male1"
    language: Annotated[str, "Language of the voice"] = "en-US"


@dataclass
class OpenVoiceTTSRequest(TTSRequest):
    language: Annotated[str, "Language of the voice"] = "EN"
    speed: Annotated[int, "Speed of speech"] = 100
    tone_color: Annotated[str, "Tone color of the voice"] = "default"
    pitch: Annotated[int, "Pitch of the voice"] = 100
    volume: Annotated[int, "Volume level"] = 100
    voice: Annotated[str, "Voice to use for speech"] = "default"



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'espeak_preferences.ui'
##
## Created by: Qt User Interface Compiler version 6.9.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QGridLayout, QGroupBox,
    QLabel, QSizePolicy, QSpacerItem, QWidget)

from airunner.gui.widgets.slider.slider_widget import SliderWidget

class Ui_espeak_preferences(object):
    def setupUi(self, espeak_preferences):
        if not espeak_preferences.objectName():
            espeak_preferences.setObjectName(u"espeak_preferences")
        espeak_preferences.resize(512, 807)
        self.gridLayout_7 = QGridLayout(espeak_preferences)
        self.gridLayout_7.setSpacing(0)
        self.gridLayout_7.setObjectName(u"gridLayout_7")
        self.gridLayout_7.setContentsMargins(0, 0, 0, 0)
        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout_7.addItem(self.verticalSpacer, 10, 0, 1, 1)

        self.espeak = QGroupBox(espeak_preferences)
        self.espeak.setObjectName(u"espeak")
        self.espeak.setCheckable(False)
        self.gridLayout_8 = QGridLayout(self.espeak)
        self.gridLayout_8.setObjectName(u"gridLayout_8")
        self.gridLayout_8.setHorizontalSpacing(0)
        self.gridLayout_8.setVerticalSpacing(10)
        self.gridLayout_8.setContentsMargins(10, 10, 10, 10)
        self.pitch = SliderWidget(self.espeak)
        self.pitch.setObjectName(u"pitch")
        self.pitch.setProperty(u"slider_minimum", 1)
        self.pitch.setProperty(u"slider_maximum", 100)
        self.pitch.setProperty(u"spinbox_minimum", 0.000000000000000)
        self.pitch.setProperty(u"spinbox_maximum", 1.000000000000000)
        self.pitch.setProperty(u"display_as_float", True)
        self.pitch.setProperty(u"slider_single_step", 1)
        self.pitch.setProperty(u"slider_page_step", 10)
        self.pitch.setProperty(u"spinbox_single_step", 0.010000000000000)
        self.pitch.setProperty(u"spinbox_page_step", 0.100000000000000)

        self.gridLayout_8.addWidget(self.pitch, 4, 0, 1, 1)

        self.groupBox_2 = QGroupBox(self.espeak)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.gridLayout_3 = QGridLayout(self.groupBox_2)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gender_combobox = QComboBox(self.groupBox_2)
        self.gender_combobox.addItem("")
        self.gender_combobox.addItem("")
        self.gender_combobox.setObjectName(u"gender_combobox")

        self.gridLayout_3.addWidget(self.gender_combobox, 0, 0, 1, 1)


        self.gridLayout_8.addWidget(self.groupBox_2, 1, 0, 1, 1)

        self.label_3 = QLabel(self.espeak)
        self.label_3.setObjectName(u"label_3")
        font = QFont()
        font.setPointSize(9)
        self.label_3.setFont(font)

        self.gridLayout_8.addWidget(self.label_3, 0, 0, 1, 1)

        self.groupBox = QGroupBox(self.espeak)
        self.groupBox.setObjectName(u"groupBox")
        self.gridLayout = QGridLayout(self.groupBox)
        self.gridLayout.setObjectName(u"gridLayout")
        self.voice_combobox = QComboBox(self.groupBox)
        self.voice_combobox.setObjectName(u"voice_combobox")

        self.gridLayout.addWidget(self.voice_combobox, 0, 0, 1, 1)


        self.gridLayout_8.addWidget(self.groupBox, 2, 0, 1, 1)

        self.volume = SliderWidget(self.espeak)
        self.volume.setObjectName(u"volume")
        self.volume.setProperty(u"slider_minimum", 1)
        self.volume.setProperty(u"slider_maximum", 100)
        self.volume.setProperty(u"spinbox_minimum", 0.000000000000000)
        self.volume.setProperty(u"spinbox_maximum", 1.000000000000000)
        self.volume.setProperty(u"display_as_float", True)
        self.volume.setProperty(u"slider_single_step", 1)
        self.volume.setProperty(u"slider_page_step", 10)
        self.volume.setProperty(u"spinbox_single_step", 0.010000000000000)
        self.volume.setProperty(u"spinbox_page_step", 0.100000000000000)

        self.gridLayout_8.addWidget(self.volume, 5, 0, 1, 1)

        self.rate = SliderWidget(self.espeak)
        self.rate.setObjectName(u"rate")
        self.rate.setProperty(u"slider_minimum", 0)
        self.rate.setProperty(u"slider_maximum", 300)
        self.rate.setProperty(u"spinbox_minimum", 0.000000000000000)
        self.rate.setProperty(u"spinbox_maximum", 3.000000000000000)
        self.rate.setProperty(u"display_as_float", True)
        self.rate.setProperty(u"slider_single_step", 1)
        self.rate.setProperty(u"slider_page_step", 10)
        self.rate.setProperty(u"spinbox_single_step", 0.010000000000000)
        self.rate.setProperty(u"spinbox_page_step", 0.100000000000000)

        self.gridLayout_8.addWidget(self.rate, 3, 0, 1, 1)


        self.gridLayout_7.addWidget(self.espeak, 0, 0, 1, 1)


        self.retranslateUi(espeak_preferences)
        self.voice_combobox.currentTextChanged.connect(espeak_preferences.voice_changed)
        self.gender_combobox.currentTextChanged.connect(espeak_preferences.gender_changed)

        QMetaObject.connectSlotsByName(espeak_preferences)
    # setupUi

    def retranslateUi(self, espeak_preferences):
        espeak_preferences.setWindowTitle(QCoreApplication.translate("espeak_preferences", u"Form", None))
        self.espeak.setTitle(QCoreApplication.translate("espeak_preferences", u"Espeak", None))
        self.pitch.setProperty(u"settings_property", QCoreApplication.translate("espeak_preferences", u"espeak_settings.pitch", None))
        self.pitch.setProperty(u"label_text", QCoreApplication.translate("espeak_preferences", u"Pitch", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("espeak_preferences", u"Gender", None))
        self.gender_combobox.setItemText(0, QCoreApplication.translate("espeak_preferences", u"Male", None))
        self.gender_combobox.setItemText(1, QCoreApplication.translate("espeak_preferences", u"Female", None))

        self.label_3.setText(QCoreApplication.translate("espeak_preferences", u"Robotic, fast, no VRAM usage", None))
        self.groupBox.setTitle(QCoreApplication.translate("espeak_preferences", u"Language", None))
        self.volume.setProperty(u"settings_property", QCoreApplication.translate("espeak_preferences", u"espeak_settings.volume", None))
        self.volume.setProperty(u"label_text", QCoreApplication.translate("espeak_preferences", u"Volume", None))
        self.rate.setProperty(u"settings_property", QCoreApplication.translate("espeak_preferences", u"espeak_settings.rate", None))
        self.rate.setProperty(u"label_text", QCoreApplication.translate("espeak_preferences", u"Rate", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'stable_diffusion_license.ui'
##
## Created by: Qt User Interface Compiler version 6.9.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QGridLayout, QLabel,
    QScrollArea, QSizePolicy, QWidget)

class Ui_stable_diffusion_license(object):
    def setupUi(self, stable_diffusion_license):
        if not stable_diffusion_license.objectName():
            stable_diffusion_license.setObjectName(u"stable_diffusion_license")
        stable_diffusion_license.resize(518, 603)
        self.gridLayout = QGridLayout(stable_diffusion_license)
        self.gridLayout.setObjectName(u"gridLayout")
        self.label = QLabel(stable_diffusion_license)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        self.label.setFont(font)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.scrollArea = QScrollArea(stable_diffusion_license)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 488, 3690))
        self.gridLayout_2 = QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.label_2 = QLabel(self.scrollAreaWidgetContents)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout_2.addWidget(self.label_2, 0, 0, 1, 1)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.gridLayout.addWidget(self.scrollArea, 1, 0, 1, 1)

        self.checkBox = QCheckBox(stable_diffusion_license)
        self.checkBox.setObjectName(u"checkBox")

        self.gridLayout.addWidget(self.checkBox, 2, 0, 1, 1)


        self.retranslateUi(stable_diffusion_license)
        self.checkBox.clicked["bool"].connect(stable_diffusion_license.agreement_clicked)

        QMetaObject.connectSlotsByName(stable_diffusion_license)
    # setupUi

    def retranslateUi(self, stable_diffusion_license):
        stable_diffusion_license.setWindowTitle(QCoreApplication.translate("stable_diffusion_license", u"Form", None))
        self.label.setText(QCoreApplication.translate("stable_diffusion_license", u"Stable Diffusion License", None))
        self.label_2.setText(QCoreApplication.translate("stable_diffusion_license", u"\n"
"Copyright (c) 2022 Robin Rombach and Patrick Esser and contributors \n"
" \n"
"CreativeML Open RAIL-M \n"
"dated August 22, 2022 \n"
" \n"
"Section I: PREAMBLE \n"
" \n"
"Multimodal generative models are being widely adopted and used, and have \n"
"the potential to transform the way artists, among other individuals, \n"
"conceive and benefit from AI or ML technologies as a tool for content \n"
"creation. \n"
" \n"
"Notwithstanding the current and potential benefits that these artifacts \n"
"can bring to society at large, there are also concerns about potential \n"
"misuses of them, either due to their technical limitations or ethical \n"
"considerations. \n"
" \n"
"In short, this license strives for both the open and responsible \n"
"downstream use of the accompanying model. When it comes to the open \n"
"character, we took inspiration from open source permissive licenses \n"
"regarding the grant of IP rights. Referring to the downstream responsible \n"
"use, we added use-based restrictions not permitting"
                        " the use of the Model \n"
"in very specific scenarios, in order for the licensor to be able to \n"
"enforce the license in case potential misuses of the Model may occur. At \n"
"the same time, we strive to promote open and responsible research on \n"
"generative models for art and content generation. \n"
" \n"
"Even though downstream derivative versions of the model could be released \n"
"under different licensing terms, the latter will always have to include - \n"
"at minimum - the same use-based restrictions as the ones in the original \n"
"license (this license). We believe in the intersection between open and \n"
"responsible AI development; thus, this License aims to strike a balance \n"
"between both in order to enable responsible open-science in the field of \n"
"AI. \n"
" \n"
"This License governs the use of the model (and its derivatives) and is \n"
"informed by the model card associated with the model. \n"
" \n"
"NOW THEREFORE, You and Licensor agree as follows: \n"
" \n"
"1. Definitions \n"
" \n"
"-"
                        " \"License\" means the terms and conditions for use, reproduction, and \n"
"Distribution as defined in this document. \n"
"- \"Data\" means a collection of information and/or content extracted from \n"
"the dataset used with the Model, including to train, pretrain, or \n"
"otherwise evaluate the Model. The Data is not licensed under this \n"
"License. \n"
"- \"Output\" means the results of operating a Model as embodied in \n"
"informational content resulting therefrom. \n"
"- \"Model\" means any accompanying machine-learning based assemblies \n"
"(including checkpoints), consisting of learnt weights, parameters \n"
"(including optimizer states), corresponding to the model architecture as \n"
"embodied in the Complementary Material, that have been trained or tuned, \n"
"in whole or in part on the Data, using the Complementary Material. \n"
"- \"Derivatives of the Model\" means all modifications to the Model, works \n"
"based on the Model, or any other model which is created or initialized by \n"
"transfer of pa"
                        "tterns of the weights, parameters, activations or output of \n"
"the Model, to the other model, in order to cause the other model to \n"
"perform similarly to the Model, including - but not limited to - \n"
"distillation methods entailing the use of intermediate data \n"
"representations or methods based on the generation of synthetic data by \n"
"the Model for training the other model. \n"
"- \"Complementary Material\" means the accompanying source code and scripts \n"
"used to define, run, load, benchmark or evaluate the Model, and used to \n"
"prepare data for training or evaluation, if any. This includes any \n"
"accompanying documentation, tutorials, examples, etc, if any. \n"
"- \"Distribution\" means any transmission, reproduction, publication or \n"
"other sharing of the Model or Derivatives of the Model to a third party, \n"
"including providing the Model as a hosted service made available by \n"
"electronic or other remote means - e.g. API-based or web access. \n"
"- \"Licensor\" means the copyright "
                        "owner or entity authorized by the \n"
"copyright owner that is granting the License, including the persons or \n"
"entities that may have rights in the Model and/or distributing the Model. \n"
"- \"You\" (or \"Your\") means an individual or Legal Entity exercising \n"
"permissions granted by this License and/or making use of the Model for \n"
"whichever purpose and in any field of use, including usage of the Model \n"
"in an end-use application - e.g. chatbot, translator, image generator. \n"
"- \"Third Parties\" means individuals or legal entities that are not under \n"
"common control with Licensor or You. \n"
"- \"Contribution\" means any work of authorship, including the original \n"
"version of the Model and any modifications or additions to that Model or \n"
"Derivatives of the Model thereof, that is intentionally submitted to \n"
"Licensor for inclusion in the Model by the copyright owner or by an \n"
"individual or Legal Entity authorized to submit on behalf of the \n"
"copyright owner. For the purpose"
                        "s of this definition, \"submitted\" means \n"
"any form of electronic, verbal, or written communication sent to the \n"
"Licensor or its representatives, including but not limited to \n"
"communication on electronic mailing lists, source code control systems, \n"
"and issue tracking systems that are managed by, or on behalf of, the \n"
"Licensor for the purpose of discussing and improving the Model, but \n"
"excluding communication that is conspicuously marked or otherwise \n"
"designated in writing by the copyright owner as \"Not a Contribution.\" \n"
"- \"Contributor\" means Licensor and any individual or Legal Entity on \n"
"behalf of whom a Contribution has been received by Licensor and \n"
"subsequently incorporated within the Model. \n"
" \n"
"Section II: INTELLECTUAL PROPERTY RIGHTS \n"
" \n"
"Both copyright and patent grants apply to the Model, Derivatives of the \n"
"Model and Complementary Material. The Model and Derivatives of the Model \n"
"are subject to additional terms as described in Section II"
                        "I. \n"
" \n"
"2. Grant of Copyright License. Subject to the terms and conditions of \n"
"this License, each Contributor hereby grants to You a perpetual, \n"
"worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright \n"
"license to reproduce, prepare, publicly display, publicly perform, \n"
"sublicense, and distribute the Complementary Material, the Model, and \n"
"Derivatives of the Model. \n"
"3. Grant of Patent License. Subject to the terms and conditions of this \n"
"License and where and as applicable, each Contributor hereby grants to \n"
"You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, \n"
"irrevocable (except as stated in this paragraph) patent license to make, \n"
"have made, use, offer to sell, sell, import, and otherwise transfer the \n"
"Model and the Complementary Material, where such license applies only to \n"
"those patent claims licensable by such Contributor that are necessarily \n"
"infringed by their Contribution(s) alone or by combination of their \n"
"C"
                        "ontribution(s) with the Model to which such Contribution(s) was \n"
"submitted. If You institute patent litigation against any entity \n"
"(including a cross-claim or counterclaim in a lawsuit) alleging that the \n"
"Model and/or Complementary Material or a Contribution incorporated within \n"
"the Model and/or Complementary Material constitutes direct or \n"
"contributory patent infringement, then any patent licenses granted to You \n"
"under this License for the Model and/or Work shall terminate as of the \n"
"date such litigation is asserted or filed. \n"
" \n"
"Section III: CONDITIONS OF USAGE, DISTRIBUTION AND REDISTRIBUTION \n"
" \n"
"4. Distribution and Redistribution. You may host for Third Party remote \n"
"access purposes (e.g. software-as-a-service), reproduce and distribute \n"
"copies of the Model or Derivatives of the Model thereof in any medium, \n"
"with or without modifications, provided that You meet the following \n"
"conditions: \n"
"Use-based restrictions as referenced in paragraph 5 MUST "
                        "be included as \n"
"an enforceable provision by You in any type of legal agreement (e.g. a \n"
"license) governing the use and/or distribution of the Model or \n"
"Derivatives of the Model, and You shall give notice to subsequent users \n"
"You Distribute to, that the Model or Derivatives of the Model are subject \n"
"to paragraph 5. This provision does not apply to the use of Complementary \n"
"Material. \n"
"You must give any Third Party recipients of the Model or Derivatives of \n"
"the Model a copy of this License; \n"
"You must cause any modified files to carry prominent notices stating that \n"
"You changed the files; \n"
"You must retain all copyright, patent, trademark, and attribution notices \n"
"excluding those notices that do not pertain to any part of the Model, \n"
"Derivatives of the Model. \n"
"You may add Your own copyright statement to Your modifications and may \n"
"provide additional or different license terms and conditions - respecting \n"
"paragraph 4.a. - for use, reproduction, or Distr"
                        "ibution of Your \n"
"modifications, or for any such Derivatives of the Model as a whole, \n"
"provided Your use, reproduction, and Distribution of the Model otherwise \n"
"complies with the conditions stated in this License. \n"
"5. Use-based restrictions. The restrictions set forth in Attachment A are \n"
"considered Use-based restrictions. Therefore You cannot use the Model and \n"
"the Derivatives of the Model for the specified restricted uses. You may \n"
"use the Model subject to this License, including only for lawful purposes \n"
"and in accordance with the License. Use may include creating any content \n"
"with, finetuning, updating, running, training, evaluating and/or \n"
"reparametrizing the Model. You shall require all of Your users who use \n"
"the Model or a Derivative of the Model to comply with the terms of this \n"
"paragraph (paragraph 5). \n"
"6. The Output You Generate. Except as set forth herein, Licensor claims \n"
"no rights in the Output You generate using the Model. You are accountable"
                        " \n"
"for the Output you generate and its subsequent uses. No use of the output \n"
"can contravene any provision as stated in the License. \n"
" \n"
"Section IV: OTHER PROVISIONS \n"
" \n"
"7. Updates and Runtime Restrictions. To the maximum extent permitted by \n"
"law, Licensor reserves the right to restrict (remotely or otherwise) \n"
"usage of the Model in violation of this License, update the Model through \n"
"electronic means, or modify the Output of the Model based on updates. You \n"
"shall undertake reasonable efforts to use the latest version of the \n"
"Model. \n"
"8. Trademarks and related. Nothing in this License permits You to make \n"
"use of Licensors\u2019 trademarks, trade names, logos or to otherwise suggest \n"
"endorsement or misrepresent the relationship between the parties; and any \n"
"rights not expressly granted herein are reserved by the Licensors. \n"
"9. Disclaimer of Warranty. Unless required by applicable law or agreed to \n"
"in writing, Licensor provides the Model and the Com"
                        "plementary Material \n"
"(and each Contributor provides its Contributions) on an \"AS IS\" BASIS, \n"
"WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, \n"
"including, without limitation, any warranties or conditions of TITLE, \n"
"NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. \n"
"You are solely responsible for determining the appropriateness of using \n"
"or redistributing the Model, Derivatives of the Model, and the \n"
"Complementary Material and assume any risks associated with Your exercise \n"
"of permissions under this License. \n"
"10. Limitation of Liability. In no event and under no legal theory, \n"
"whether in tort (including negligence), contract, or otherwise, unless \n"
"required by applicable law (such as deliberate and grossly negligent \n"
"acts) or agreed to in writing, shall any Contributor be liable to You for \n"
"damages, including any direct, indirect, special, incidental, or \n"
"consequential damages of any character arising as a resu"
                        "lt of this \n"
"License or out of the use or inability to use the Model and the \n"
"Complementary Material (including but not limited to damages for loss of \n"
"goodwill, work stoppage, computer failure or malfunction, or any and all \n"
"other commercial damages or losses), even if such Contributor has been \n"
"advised of the possibility of such damages. \n"
"11. Accepting Warranty or Additional Liability. While redistributing the \n"
"Model, Derivatives of the Model and the Complementary Material thereof, \n"
"You may choose to offer, and charge a fee for, acceptance of support, \n"
"warranty, indemnity, or other liability obligations and/or rights \n"
"consistent with this License. However, in accepting such obligations, You \n"
"may act only on Your own behalf and on Your sole responsibility, not on \n"
"behalf of any other Contributor, and only if You agree to indemnify, \n"
"defend, and hold each Contributor harmless for any liability incurred by, \n"
"or claims asserted against, such Contributor by r"
                        "eason of your accepting \n"
"any such warranty or additional liability. \n"
"12. If any provision of this License is held to be invalid, illegal or \n"
"unenforceable, the remaining provisions shall be unaffected thereby and \n"
"remain valid as if such provision had not been set forth herein. \n"
" \n"
"", None))
        self.checkBox.setText(QCoreApplication.translate("stable_diffusion_license", u"I have read and agree to the terms of the Stable Diffusion license", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'active_grid_settings.ui'
##
## Created by: Qt User Interface Compiler version 6.9.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QFrame, QGridLayout,
    QGroupBox, QHBoxLayout, QLabel, QPushButton,
    QScrollArea, QSizePolicy, QSpacerItem, QWidget)

from airunner.gui.widgets.slider.slider_widget import SliderWidget

class Ui_active_grid_settings_widget(object):
    def setupUi(self, active_grid_settings_widget):
        if not active_grid_settings_widget.objectName():
            active_grid_settings_widget.setObjectName(u"active_grid_settings_widget")
        active_grid_settings_widget.resize(724, 369)
        self.gridLayout = QGridLayout(active_grid_settings_widget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setHorizontalSpacing(0)
        self.gridLayout.setVerticalSpacing(10)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.scrollArea = QScrollArea(active_grid_settings_widget)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setFrameShape(QFrame.Shape.NoFrame)
        self.scrollArea.setFrameShadow(QFrame.Shadow.Plain)
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 724, 296))
        self.gridLayout_3 = QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gridLayout_3.setHorizontalSpacing(0)
        self.gridLayout_3.setVerticalSpacing(10)
        self.gridLayout_3.setContentsMargins(0, 0, 10, 0)
        self.groupBox_3 = QGroupBox(self.scrollAreaWidgetContents)
        self.groupBox_3.setObjectName(u"groupBox_3")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
        self.groupBox_3.setSizePolicy(sizePolicy)
        self.groupBox_3.setMinimumSize(QSize(0, 0))
        self.groupBox_3.setFlat(False)
        self.horizontalLayout = QHBoxLayout(self.groupBox_3)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.width_slider_widget = SliderWidget(self.groupBox_3)
        self.width_slider_widget.setObjectName(u"width_slider_widget")
        self.width_slider_widget.setMinimumSize(QSize(0, 20))
        self.width_slider_widget.setProperty(u"slider_minimum", 64)
        self.width_slider_widget.setProperty(u"slider_maximum", 4096)
        self.width_slider_widget.setProperty(u"spinbox_minimum", 64)
        self.width_slider_widget.setProperty(u"spinbox_maximum", 4096)
        self.width_slider_widget.setProperty(u"slider_tick_interval", 64)
        self.width_slider_widget.setProperty(u"slider_single_step", 64)
        self.width_slider_widget.setProperty(u"slider_page_step", 64)
        self.width_slider_widget.setProperty(u"spinbox_single_step", 64)
        self.width_slider_widget.setProperty(u"spinbox_page_step", 64)
        self.width_slider_widget.setProperty(u"settings_property", u"application_settings.working_width")
        self.width_slider_widget.setProperty(u"display_as_float", False)

        self.horizontalLayout.addWidget(self.width_slider_widget)

        self.size_lock_button = QPushButton(self.groupBox_3)
        self.size_lock_button.setObjectName(u"size_lock_button")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.size_lock_button.sizePolicy().hasHeightForWidth())
        self.size_lock_button.setSizePolicy(sizePolicy1)
        icon = QIcon(QIcon.fromTheme(u"system-lock-screen"))
        self.size_lock_button.setIcon(icon)
        self.size_lock_button.setCheckable(True)

        self.horizontalLayout.addWidget(self.size_lock_button)

        self.height_slider_widget = SliderWidget(self.groupBox_3)
        self.height_slider_widget.setObjectName(u"height_slider_widget")
        self.height_slider_widget.setMinimumSize(QSize(0, 20))
        self.height_slider_widget.setProperty(u"slider_minimum", 64)
        self.height_slider_widget.setProperty(u"slider_maximum", 4096)
        self.height_slider_widget.setProperty(u"spinbox_minimum", 64)
        self.height_slider_widget.setProperty(u"spinbox_maximum", 4096)
        self.height_slider_widget.setProperty(u"slider_tick_interval", 64)
        self.height_slider_widget.setProperty(u"slider_single_step", 64)
        self.height_slider_widget.setProperty(u"slider_page_step", 64)
        self.height_slider_widget.setProperty(u"spinbox_single_step", 64)
        self.height_slider_widget.setProperty(u"spinbox_page_step", 64)
        self.height_slider_widget.setProperty(u"display_as_float", False)

        self.horizontalLayout.addWidget(self.height_slider_widget)


        self.gridLayout_3.addWidget(self.groupBox_3, 0, 0, 1, 1)

        self.active_grid_border_groupbox = QGroupBox(self.scrollAreaWidgetContents)
        self.active_grid_border_groupbox.setObjectName(u"active_grid_border_groupbox")
        sizePolicy.setHeightForWidth(self.active_grid_border_groupbox.sizePolicy().hasHeightForWidth())
        self.active_grid_border_groupbox.setSizePolicy(sizePolicy)
        self.active_grid_border_groupbox.setMinimumSize(QSize(0, 0))
        self.active_grid_border_groupbox.setCheckable(True)
        self.active_grid_border_groupbox.setChecked(False)
        self.horizontalLayout_3 = QHBoxLayout(self.active_grid_border_groupbox)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.border_opacity_slider_widget = SliderWidget(self.active_grid_border_groupbox)
        self.border_opacity_slider_widget.setObjectName(u"border_opacity_slider_widget")
        self.border_opacity_slider_widget.setProperty(u"spinbox_single_step", 0.010000000000000)
        self.border_opacity_slider_widget.setProperty(u"slider_page_step", 1)
        self.border_opacity_slider_widget.setProperty(u"spinbox_page_step", 0.100000000000000)
        self.border_opacity_slider_widget.setProperty(u"display_as_float", True)
        self.border_opacity_slider_widget.setProperty(u"slider_minimum", 0)
        self.border_opacity_slider_widget.setProperty(u"slider_maximum", 255)
        self.border_opacity_slider_widget.setProperty(u"spinbox_minimum", 0.000000000000000)
        self.border_opacity_slider_widget.setProperty(u"spinbox_maximum", 1.000000000000000)
        self.border_opacity_slider_widget.setProperty(u"slider_tick_interval", 1.000000000000000)
        self.border_opacity_slider_widget.setProperty(u"slider_singlestep", 1)

        self.horizontalLayout_3.addWidget(self.border_opacity_slider_widget)

        self.border_choose_color_button = QPushButton(self.active_grid_border_groupbox)
        self.border_choose_color_button.setObjectName(u"border_choose_color_button")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Preferred)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.border_choose_color_button.sizePolicy().hasHeightForWidth())
        self.border_choose_color_button.setSizePolicy(sizePolicy2)
        self.border_choose_color_button.setCursor(QCursor(Qt.CursorShape.PointingHandCursor))

        self.horizontalLayout_3.addWidget(self.border_choose_color_button)


        self.gridLayout_3.addWidget(self.active_grid_border_groupbox, 1, 0, 1, 1)

        self.active_grid_fill_groupbox = QGroupBox(self.scrollAreaWidgetContents)
        self.active_grid_fill_groupbox.setObjectName(u"active_grid_fill_groupbox")
        sizePolicy.setHeightForWidth(self.active_grid_fill_groupbox.sizePolicy().hasHeightForWidth())
        self.active_grid_fill_groupbox.setSizePolicy(sizePolicy)
        self.active_grid_fill_groupbox.setMinimumSize(QSize(0, 0))
        self.active_grid_fill_groupbox.setCheckable(True)
        self.active_grid_fill_groupbox.setChecked(False)
        self.horizontalLayout_2 = QHBoxLayout(self.active_grid_fill_groupbox)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.fill_opacity_slider_widget = SliderWidget(self.active_grid_fill_groupbox)
        self.fill_opacity_slider_widget.setObjectName(u"fill_opacity_slider_widget")
        self.fill_opacity_slider_widget.setProperty(u"spinbox_single_step", 0.010000000000000)
        self.fill_opacity_slider_widget.setProperty(u"slider_page_step", 1)
        self.fill_opacity_slider_widget.setProperty(u"spinbox_page_step", 0.100000000000000)
        self.fill_opacity_slider_widget.setProperty(u"display_as_float", True)
        self.fill_opacity_slider_widget.setProperty(u"slider_minimum", 0)
        self.fill_opacity_slider_widget.setProperty(u"slider_maximum", 255)
        self.fill_opacity_slider_widget.setProperty(u"spinbox_minimum", 0.000000000000000)
        self.fill_opacity_slider_widget.setProperty(u"spinbox_maximum", 1.000000000000000)
        self.fill_opacity_slider_widget.setProperty(u"slider_tick_interval", 1.000000000000000)
        self.fill_opacity_slider_widget.setProperty(u"slider_singlestep", 1)

        self.horizontalLayout_2.addWidget(self.fill_opacity_slider_widget)

        self.fill_choose_color_button = QPushButton(self.active_grid_fill_groupbox)
        self.fill_choose_color_button.setObjectName(u"fill_choose_color_button")
        sizePolicy2.setHeightForWidth(self.fill_choose_color_button.sizePolicy().hasHeightForWidth())
        self.fill_choose_color_button.setSizePolicy(sizePolicy2)
        self.fill_choose_color_button.setCursor(QCursor(Qt.CursorShape.PointingHandCursor))

        self.horizontalLayout_2.addWidget(self.fill_choose_color_button)


        self.gridLayout_3.addWidget(self.active_grid_fill_groupbox, 2, 0, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 139, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout_3.addItem(self.verticalSpacer, 3, 0, 1, 1)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.gridLayout.addWidget(self.scrollArea, 3, 0, 1, 1)

        self.label = QLabel(active_grid_settings_widget)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setBold(True)
        self.label.setFont(font)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.active_grid_area_checkbox = QCheckBox(active_grid_settings_widget)
        self.active_grid_area_checkbox.setObjectName(u"active_grid_area_checkbox")
        font1 = QFont()
        font1.setBold(False)
        self.active_grid_area_checkbox.setFont(font1)

        self.gridLayout.addWidget(self.active_grid_area_checkbox, 2, 0, 1, 1)

        self.line = QFrame(active_grid_settings_widget)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line, 1, 0, 1, 1)


        self.retranslateUi(active_grid_settings_widget)
        self.active_grid_area_checkbox.clicked["bool"].connect(active_grid_settings_widget.action_clicked_checkbox_toggle_active_grid_area)
        self.size_lock_button.toggled.connect(active_grid_settings_widget.size_lock_toggled)
        self.border_choose_color_button.clicked.connect(active_grid_settings_widget.action_choose_border_color_clicked)
        self.fill_choose_color_button.clicked.connect(active_grid_settings_widget.action_choose_fill_color_clicked)

        QMetaObject.connectSlotsByName(active_grid_settings_widget)
    # setupUi

    def retranslateUi(self, active_grid_settings_widget):
        active_grid_settings_widget.setWindowTitle(QCoreApplication.translate("active_grid_settings_widget", u"Form", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("active_grid_settings_widget", u"Size", None))
        self.width_slider_widget.setProperty(u"label_text", QCoreApplication.translate("active_grid_settings_widget", u"Width", None))
        self.size_lock_button.setText("")
        self.height_slider_widget.setProperty(u"label_text", QCoreApplication.translate("active_grid_settings_widget", u"Height", None))
        self.height_slider_widget.setProperty(u"settings_property", QCoreApplication.translate("active_grid_settings_widget", u"application_settings.working_height", None))
        self.active_grid_border_groupbox.setTitle(QCoreApplication.translate("active_grid_settings_widget", u"Border", None))
        self.border_opacity_slider_widget.setProperty(u"label_text", QCoreApplication.translate("active_grid_settings_widget", u"Border Opacity", None))
        self.border_opacity_slider_widget.setProperty(u"settings_property", QCoreApplication.translate("active_grid_settings_widget", u"active_grid_settings.border_opacity", None))
        self.border_choose_color_button.setText(QCoreApplication.translate("active_grid_settings_widget", u"Color", None))
        self.active_grid_fill_groupbox.setTitle(QCoreApplication.translate("active_grid_settings_widget", u"Fill", None))
        self.fill_opacity_slider_widget.setProperty(u"label_text", QCoreApplication.translate("active_grid_settings_widget", u"Fill Opacity", None))
        self.fill_opacity_slider_widget.setProperty(u"settings_property", QCoreApplication.translate("active_grid_settings_widget", u"active_grid_settings.fill_opacity", None))
        self.fill_choose_color_button.setText(QCoreApplication.translate("active_grid_settings_widget", u"Color", None))
        self.label.setText(QCoreApplication.translate("active_grid_settings_widget", u"Active Grid Settings", None))
        self.active_grid_area_checkbox.setText(QCoreApplication.translate("active_grid_settings_widget", u"Enable Active Grid Area", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'choose_style.ui'
##
## Created by: Qt User Interface Compiler version 6.9.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QFrame, QGridLayout,
    QLabel, QSizePolicy, QSpacerItem, QWidget)

class Ui_choose_model_style(object):
    def setupUi(self, choose_model_style):
        if not choose_model_style.objectName():
            choose_model_style.setObjectName(u"choose_model_style")
        choose_model_style.resize(530, 533)
        self.gridLayout = QGridLayout(choose_model_style)
        self.gridLayout.setObjectName(u"gridLayout")
        self.frame_3 = QFrame(choose_model_style)
        self.frame_3.setObjectName(u"frame_3")
        self.frame_3.setFrameShape(QFrame.StyledPanel)
        self.frame_3.setFrameShadow(QFrame.Raised)
        self.gridLayout_4 = QGridLayout(self.frame_3)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.label_3 = QLabel(self.frame_3)
        self.label_3.setObjectName(u"label_3")
        font = QFont()
        font.setBold(True)
        self.label_3.setFont(font)

        self.gridLayout_4.addWidget(self.label_3, 0, 0, 1, 1)

        self.comboBox = QComboBox(self.frame_3)
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.setObjectName(u"comboBox")

        self.gridLayout_4.addWidget(self.comboBox, 1, 0, 1, 1)


        self.gridLayout.addWidget(self.frame_3, 5, 0, 1, 1)

        self.label_2 = QLabel(choose_model_style)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setWordWrap(True)

        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)

        self.label = QLabel(choose_model_style)
        self.label.setObjectName(u"label")
        font1 = QFont()
        font1.setPointSize(11)
        font1.setBold(True)
        self.label.setFont(font1)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 10, 0, 1, 1)

        self.line = QFrame(choose_model_style)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line, 2, 0, 1, 1)


        self.retranslateUi(choose_model_style)

        QMetaObject.connectSlotsByName(choose_model_style)
    # setupUi

    def retranslateUi(self, choose_model_style):
        choose_model_style.setWindowTitle(QCoreApplication.translate("choose_model_style", u"Form", None))
        self.label_3.setText(QCoreApplication.translate("choose_model_style", u"Style", None))
        self.comboBox.setItemText(0, QCoreApplication.translate("choose_model_style", u"Photography", None))
        self.comboBox.setItemText(1, QCoreApplication.translate("choose_model_style", u"Illustration", None))

        self.label_2.setText(QCoreApplication.translate("choose_model_style", u"Choose your default model settings. These can be overriden at any time.", None))
        self.label.setText(QCoreApplication.translate("choose_model_style", u"Stable Diffusion Model Setup", None))
    # retranslateUi




from PySide6.QtCore import Slot
from airunner.gui.widgets.base_widget import BaseWidget
from airunner.gui.widgets.stablediffusion.templates.stablediffusion_tool_tab_ui import (
    Ui_stablediffusion_tool_tab_widget,
)
from airunner.data.models import Tab


class StablediffusionToolTabWidget(BaseWidget):
    widget_class_ = Ui_stablediffusion_tool_tab_widget

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.ui.tool_tab_widget_container.currentChanged.connect(
            self.on_tool_tab_widget_container_currentChanged
        )

    @Slot(int)
    def on_tool_tab_widget_container_currentChanged(self, index: int):
        Tab.update_tabs(
            "stablediffusion_tool_tab",
            self.ui.tool_tab_widget_container,
            index,
        )

    def showEvent(self, event):
        super().showEvent(event)
        section = "stablediffusion_tool_tab"
        for index in range(self.ui.tool_tab_widget_container.count()):
            tab_text = self.ui.tool_tab_widget_container.tabText(index)
            tab = Tab.objects.filter_by_first(section=section, name=tab_text)
            if not tab:
                Tab.objects.create(
                    section=section, name=tab_text, index=index, active=False
                )

        tabs = Tab.objects.filter_by(section=section)
        for tab in tabs:
            if tab.active:
                self.ui.tool_tab_widget_container.setCurrentIndex(tab.index)
                break



import unittest
import os
from unittest.mock import patch, MagicMock, mock_open
import tempfile
import shutil

from airunner.utils.models.scan_path_for_items import (
    scan_path_for_lora,
    scan_path_for_embeddings,
)
from airunner.data.models import Lora, Embedding


class TestScanPathForItems(unittest.TestCase):
    """Test cases for scanning paths for Lora and Embedding items."""

    def setUp(self):
        """Set up the test environment."""
        # Create a temporary directory structure for testing
        self.temp_dir = tempfile.mkdtemp()

        # Mock paths for test data
        self.base_path = self.temp_dir
        self.models_path = os.path.join(self.base_path, "art", "models")
        self.version_dir = os.path.join(self.models_path, "v1.0")
        self.lora_path = os.path.join(self.version_dir, "lora")
        self.embeddings_path = os.path.join(self.version_dir, "embeddings")

        # Create directory structure
        try:
            os.makedirs(self.lora_path, exist_ok=True)
        except FileExistsError:
            pass
        try:
            os.makedirs(self.embeddings_path, exist_ok=True)
        except FileExistsError:
            pass

    def tearDown(self):
        """Clean up after tests."""
        # Remove temporary directory and all contents
        shutil.rmtree(self.temp_dir)

    @patch("airunner.utils.models.scan_path_for_items.SettingsMixin")
    def test_scan_path_for_lora_no_files(self, mock_settings_mixin):
        """Test scanning for Lora models when no files exist."""
        # Setup mock for database operations
        mock_database = MagicMock()
        mock_database.session.query.return_value.all.return_value = []
        mock_settings_mixin.return_value = mock_database

        # Run the scan
        scan_result = scan_path_for_lora(self.base_path)

        # Verify no changes were made (no files to process)
        self.assertFalse(scan_result)
        mock_database.session.commit.assert_not_called()

    @patch("airunner.utils.models.scan_path_for_items.SettingsMixin")
    def test_scan_path_for_lora_new_file(self, mock_settings_mixin):
        """Test scanning for Lora models when new file is found."""
        # Create a test file
        lora_file_path = os.path.join(self.lora_path, "test_lora.safetensors")
        with open(lora_file_path, "w") as f:
            f.write("test content")

        # Setup mock for database operations
        mock_database = MagicMock()
        mock_database.session.query.return_value.all.return_value = []
        mock_database.get_lora_by_name.return_value = None
        mock_settings_mixin.return_value = mock_database

        # Run the scan
        scan_result = scan_path_for_lora(self.base_path)

        # Verify a new Lora was added
        self.assertTrue(scan_result)
        mock_database.session.add.assert_called_once()
        mock_database.session.commit.assert_called_once()

        # Verify the correct Lora object was created
        added_lora = mock_database.session.add.call_args[0][0]
        self.assertEqual(added_lora.name, "test_lora")
        self.assertEqual(added_lora.path, lora_file_path)
        self.assertEqual(added_lora.version, "v1.0")
        self.assertEqual(added_lora.scale, 1)
        self.assertFalse(added_lora.enabled)
        self.assertFalse(added_lora.loaded)
        self.assertEqual(added_lora.trigger_word, "")

    @patch("airunner.utils.models.scan_path_for_items.SettingsMixin")
    def test_scan_path_for_lora_delete_missing(self, mock_settings_mixin):
        """Test scanning for Lora models when a file is missing."""
        # Setup mock for database operations
        missing_lora = Lora(
            name="missing_lora",
            path="/nonexistent/path/missing_lora.safetensors",
            scale=1,
            enabled=True,
            loaded=True,
            trigger_word="test",
            version="v1.0",
        )

        mock_database = MagicMock()
        mock_database.session.query.return_value.all.return_value = [
            missing_lora
        ]
        mock_settings_mixin.return_value = mock_database

        # Run the scan
        scan_result = scan_path_for_lora(self.base_path)

        # Verify the missing Lora was deleted
        self.assertTrue(scan_result)
        mock_database.session.delete.assert_called_once_with(missing_lora)
        mock_database.session.commit.assert_called_once()

    @patch("airunner.utils.models.scan_path_for_items.SettingsMixin")
    def test_scan_path_for_lora_update_existing(self, mock_settings_mixin):
        """Test scanning for Lora models when a file needs to be updated."""
        # Create a test file
        lora_file_path = os.path.join(self.lora_path, "test_lora.safetensors")
        with open(lora_file_path, "w") as f:
            f.write("test content")

        # Create an existing Lora with different path
        existing_lora = Lora(
            name="test_lora",
            path="/old/path/test_lora.safetensors",
            scale=1,
            enabled=True,
            loaded=True,
            trigger_word="test",
            version="v0.9",
        )

        # Setup mock for database operations
        mock_database = MagicMock()
        mock_database.session.query.return_value.all.return_value = [
            existing_lora
        ]
        mock_database.get_lora_by_name.return_value = existing_lora
        mock_settings_mixin.return_value = mock_database

        # Run the scan
        scan_result = scan_path_for_lora(self.base_path)

        # Verify a new Lora was added (since path changed)
        self.assertTrue(scan_result)
        mock_database.session.add.assert_called_once()
        mock_database.session.commit.assert_called_once()

    @patch("airunner.utils.models.scan_path_for_items.SettingsMixin")
    def test_scan_path_for_embeddings_new_file(self, mock_settings_mixin):
        """Test scanning for embeddings when a new file is found."""
        # Create a test file
        embedding_file_path = os.path.join(
            self.embeddings_path, "test_embedding.pt"
        )
        with open(embedding_file_path, "w") as f:
            f.write("test content")

        # Setup mock for database operations
        mock_database = MagicMock()
        mock_database.session.query.return_value.all.return_value = []
        mock_database.get_embedding_by_name.return_value = None
        mock_settings_mixin.return_value = mock_database

        # Run the scan
        scan_result = scan_path_for_embeddings(self.base_path)

        # Verify a new embedding was added
        self.assertTrue(scan_result)
        mock_database.session.add.assert_called_once()
        mock_database.session.commit.assert_called_once()

        # Verify the correct Embedding object was created
        added_embedding = mock_database.session.add.call_args[0][0]
        self.assertEqual(added_embedding.name, "test_embedding")
        self.assertEqual(added_embedding.path, embedding_file_path)
        self.assertEqual(added_embedding.version, "v1.0")
        self.assertEqual(added_embedding.tags, "")
        self.assertFalse(added_embedding.active)
        self.assertEqual(added_embedding.trigger_word, "")

    @patch("airunner.utils.models.scan_path_for_items.SettingsMixin")
    def test_scan_path_for_embeddings_delete_missing(
        self, mock_settings_mixin
    ):
        """Test scanning for embeddings when a file is missing."""
        # Setup mock for database operations
        missing_embedding = Embedding(
            name="missing_embedding",
            path="/nonexistent/path/missing_embedding.pt",
            version="v1.0",
            tags="test",
            active=True,
            trigger_word="test",
        )

        mock_database = MagicMock()
        mock_database.session.query.return_value.all.return_value = [
            missing_embedding
        ]
        mock_settings_mixin.return_value = mock_database

        # Run the scan
        scan_result = scan_path_for_embeddings(self.base_path)

        # Verify the missing embedding was deleted
        self.assertTrue(scan_result)
        mock_database.session.delete.assert_called_once_with(missing_embedding)
        mock_database.session.commit.assert_called_once()

    @patch("airunner.utils.models.scan_path_for_items.SettingsMixin")
    def test_scan_path_for_multiple_file_extensions(self, mock_settings_mixin):
        """Test scanning with multiple file extensions."""
        # Create test files with different extensions
        lora_file1 = os.path.join(self.lora_path, "test_lora1.safetensors")
        lora_file2 = os.path.join(self.lora_path, "test_lora2.ckpt")
        lora_file3 = os.path.join(self.lora_path, "test_lora3.pt")
        lora_file4 = os.path.join(
            self.lora_path, "test_lora4.txt"
        )  # Should be ignored

        for file_path in [lora_file1, lora_file2, lora_file3, lora_file4]:
            with open(file_path, "w") as f:
                f.write("test content")

        # Setup mock for database operations
        mock_database = MagicMock()
        mock_database.session.query.return_value.all.return_value = []
        mock_database.get_lora_by_name.return_value = None
        mock_settings_mixin.return_value = mock_database

        # Run the scan
        scan_result = scan_path_for_lora(self.base_path)

        # Verify the correct files were processed (3 valid extensions, 1 ignored)
        self.assertTrue(scan_result)
        self.assertEqual(mock_database.session.add.call_count, 3)
        mock_database.session.commit.assert_called_once()


if __name__ == "__main__":
    unittest.main()
